# Copyright 2017 OpenStack Foundation
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

"""add default onboarded template source

Revision ID: ef14f8026327
Revises: e8918cda6433
Create Date: 2017-02-10 12:10:09.606460

"""

# flake8: noqa: E402

# revision identifiers, used by Alembic.
revision = 'ef14f8026327'
down_revision = 'e8918cda6433'

from alembic import op


def upgrade(active_plugins=None, options=None):
    op.alter_column('vnfd', 'template_source',
                    server_default="onboarded")

    op.execute("UPDATE vnfd set template_source='onboarded'"
       " WHERE template_source is NULL")



#  Licensed under the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.

import compiler
import re


def is_log_callfunc(n):
    """LOG.xxx('hello %s' % xyz) and LOG('hello')"""
    if isinstance(n.parent, compiler.ast.Mod):
        n = n.parent
    if isinstance(n.parent, compiler.ast.CallFunc):
        if isinstance(n.parent.node, compiler.ast.Getattr):
            if isinstance(n.parent.node.getChildNodes()[0],
                          compiler.ast.Name):
                if n.parent.node.getChildNodes()[0].name == 'LOG':
                    return True
    return False


def is_log_i18n_msg_with_mod(n):
    """LOG.xxx("Hello %s" % xyz) should be LOG.xxx("Hello %s", xyz)"""
    if not isinstance(n.parent.parent, compiler.ast.Mod):
        return False
    n = n.parent.parent
    if isinstance(n.parent, compiler.ast.CallFunc):
        if isinstance(n.parent.node, compiler.ast.Getattr):
            if isinstance(n.parent.node.getChildNodes()[0],
                          compiler.ast.Name):
                if n.parent.node.getChildNodes()[0].name == 'LOG':
                    return True
    return False


def is_wrong_i18n_format(n):
    """Check _('hello %s' % xyz)"""
    if isinstance(n.parent, compiler.ast.Mod):
        n = n.parent
    if isinstance(n.parent, compiler.ast.CallFunc):
        if isinstance(n.parent.node, compiler.ast.Name):
            if n.parent.node.name == '_':
                return True
    return False


"""
Used for check message need be localized or not.
(predicate_func, action, message)
"""
i18n_msg_predicates = [
    # Skip ['hello world', 1]
    (lambda n: isinstance(n.parent, compiler.ast.List), 'skip', ''),
    # Skip {'hellow world', 1}
    (lambda n: isinstance(n.parent, compiler.ast.Dict), 'skip', ''),
    # Skip msg['hello world']
    (lambda n: isinstance(n.parent, compiler.ast.Subscript), 'skip', ''),
    # Skip doc string
    (lambda n: isinstance(n.parent, compiler.ast.Discard), 'skip', ''),
    # Skip msg = "hello", in normal, message should more than one word
    (lambda n: len(n.value.strip().split(' ')) <= 1, 'skip', ''),
    # Skip msg = 'hello world' + vars + 'world hello'
    (lambda n: isinstance(n.parent, compiler.ast.Add), 'skip', ''),
    # Skip xml markers msg = "<test></test>"
    (lambda n: len(re.compile("</.*>").findall(n.value)) > 0, 'skip', ''),
    # Skip sql statement
    (lambda n: len(
        re.compile("^SELECT.*FROM", flags=re.I).findall(n.value)) > 0,
     'skip', ''),
    # LOG.xxx()
    (is_log_callfunc, 'error', 'Message must be localized'),
    # _('hello %s' % xyz) should be _('hello %s') % xyz
    (is_wrong_i18n_format, 'error',
     ("Message format was wrong, _('hello %s' % xyz) "
      "should be _('hello %s') % xyz")),
    # default
    (lambda n: True, 'warn', 'Message might need localized')
]


"""
Used for checking message format. (checker_func, message)
"""
msg_format_checkers = [
    # If message contain more than on format specifier, it should use
    # mapping key
    (lambda n: len(re.compile("%[bcdeEfFgGnosxX]").findall(n.value)) > 1,
     "The message shouldn't contain more than one format specifier"),
    # Check capital
    (lambda n: n.value.split(' ')[0].count('_') == 0 and
     n.value[0].isalpha() and
     n.value[0].islower(),
     "First letter must be capital"),
    (is_log_i18n_msg_with_mod,
     'LOG.xxx("Hello %s" % xyz) should be LOG.xxx("Hello %s", xyz)')
]


file_black_list = ["./tacker/tests/unit",
                   "./tacker/openstack",
                   "./tacker/plugins/bigswitch/tests"]



# Copyright (C) 2020 FUJITSU
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import datetime

from kubernetes import client
from oslo_serialization import jsonutils
from oslo_utils import uuidutils
from tacker.db.db_sqlalchemy import models
from tacker import objects
from tacker.objects import vim_connection
from tacker.tests import uuidsentinel

CREATE_K8S_FALSE_VALUE = None


def fake_k8s_dict():
    k8s_client_dict = {
        'namespace': 'curryns',
        'object': fake_k8s_obj()
    }
    return k8s_client_dict


def fake_k8s_obj():
    return client.V1Deployment(
        api_version='apps/v1',
        kind='Deployment',
        metadata=client.V1ObjectMeta(
            name='curry-test001',
            namespace='curryns'
        ),
        spec=client.V1DeploymentSpec(
            replicas=2,
            selector=client.V1LabelSelector(
                match_labels={'app': 'webserver'}
            ),
            template=client.V1PodTemplateSpec(
                metadata=client.V1ObjectMeta(
                    labels={'app': 'webserver',
                            'scaling_name': 'SP1'}
                ),
                spec=client.V1PodSpec(
                    containers=[
                        client.V1Container(
                            env=[
                                client.V1EnvVar(
                                    name='param0',
                                    value_from=client.V1EnvVarSource(
                                        config_map_key_ref=client.
                                        V1ConfigMapKeySelector(
                                            key='param0',
                                            name='curry-test001'
                                        )
                                    )
                                ),
                                client.V1EnvVar(
                                    name='param1',
                                    value_from=client.V1EnvVarSource(
                                        config_map_key_ref=client.
                                        V1ConfigMapKeySelector(
                                            key='param1',
                                            name='curry-test001'
                                        )
                                    )
                                )
                            ],
                            image='celebdor/kuryr-demo',
                            image_pull_policy='IfNotPresent',
                            name='web-server',
                            ports=[
                                client.V1ContainerPort(
                                    container_port=8080
                                )
                            ],
                            resources=client.V1ResourceRequirements(
                                limits={
                                    'cpu': '500m', 'memory': '512M'
                                },
                                requests={
                                    'cpu': '500m', 'memory': '512M'
                                }
                            ),
                            volume_mounts=[
                                client.V1VolumeMount(
                                    name='curry-claim-volume',
                                    mount_path='/data'
                                )
                            ]
                        )
                    ],
                    volumes=[
                        client.V1Volume(
                            name='curry-claim-volume',
                            persistent_volume_claim=client.
                            V1PersistentVolumeClaimVolumeSource(
                                claim_name='curry-pv-claim'
                            )
                        )
                    ],
                    termination_grace_period_seconds=0
                )
            )
        )
    )


def fake_k8s_client_dict():
    k8s_client_dict = {
        'v1': client.CoreV1Api(),
        'apiregistration.k8s.io/v1': client.ApiregistrationV1Api(),
        'apps/v1': client.AppsV1Api(),
        'authentication.k8s.io/v1': client.AuthenticationV1Api(),
        'authorization.k8s.io/v1': client.AuthorizationV1Api(),
        'autoscaling/v1': client.AutoscalingV1Api(),
        'batch/v1': client.BatchV1Api(),
        'coordination.k8s.io/v1': client.CoordinationV1Api(),
        'networking.k8s.io/v1': client.NetworkingV1Api(),
        'rbac.authorization.k8s.io/v1': client.RbacAuthorizationV1Api(),
        'scheduling.k8s.io/v1': client.SchedulingV1Api(),
        'storage.k8s.io/v1': client.StorageV1Api()
    }
    return k8s_client_dict


def fake_k8s_objs_node():
    objs = [
        {
            'namespace': 'test',
            'status': 'Creating',
            'object': fake_node()
        }
    ]
    return objs


def fake_node():
    return client.V1Node(
        api_version='v1',
        kind='Node',
        metadata=client.V1ObjectMeta(
            name='curry-node-test',
            labels={'name': 'curry-node-test'}
        ),
        status=client.V1NodeStatus(
            conditions=[
                client.V1NodeCondition(
                    status='True',
                    type='Ready'
                )
            ]
        )
    )


def fake_k8s_objs_node_status_false():
    objs = [
        {
            'namespace': 'test',
            'status': 'Creating',
            'object': fake_node_false()
        }
    ]
    return objs


def fake_node_false():
    return client.V1Node(
        api_version='v1',
        kind='Node',
        metadata=client.V1ObjectMeta(
            name='curry-node-test',
            labels={'name': 'curry-node-test'}
        ),
        status=client.V1NodeStatus(
            conditions=[
                client.V1NodeCondition(
                    status='False',
                    type='Ready'
                )
            ]
        )
    )


def fake_k8s_objs_pvc():
    objs = [
        {
            'namespace': 'test',
            'status': 'Creating',
            'object': fake_pvc()
        }
    ]
    return objs


def fake_pvc():
    return client.V1PersistentVolumeClaim(
        api_version='v1',
        kind='PersistentVolumeClaim',
        metadata=client.V1ObjectMeta(
            name='curry-sc-pvc'
        ),
        status=client.V1PersistentVolumeClaimStatus(
            phase='Bound'
        )
    )


def fake_k8s_objs_pvc_false_phase():
    objs = [
        {
            'namespace': 'test',
            'status': 'Creating',
            'object': fake_pvc_false()
        }
    ]
    return objs


def fake_pvc_false():
    return client.V1PersistentVolumeClaim(
        api_version='v1',
        kind='PersistentVolumeClaim',
        metadata=client.V1ObjectMeta(
            name='curry-sc-pvc'
        ),
        status=client.V1PersistentVolumeClaimStatus(
            phase='UnBound'
        )
    )


def fake_k8s_objs_namespace():
    objs = [
        {
            'namespace': 'test',
            'status': 'Creating',
            'object': fake_namespace()
        }
    ]
    return objs


def fake_namespace():
    return client.V1Namespace(
        api_version='v1',
        kind='Namespace',
        metadata=client.V1ObjectMeta(
            name='curry-ns'
        ),
        status=client.V1NamespaceStatus(
            phase='Active'
        )
    )


def fake_k8s_objs_namespace_false_phase():
    objs = [
        {
            'namespace': 'test',
            'status': 'Creating',
            'object': fake_namespace_false()
        }
    ]
    return objs


def fake_namespace_false():
    return client.V1Namespace(
        api_version='v1',
        kind='Namespace',
        metadata=client.V1ObjectMeta(
            name='curry-ns'
        ),
        status=client.V1NamespaceStatus(
            phase='NotActive'
        )
    )


def fake_k8s_objs_service():
    objs = [
        {
            'namespace': 'default',
            'status': 'Creating',
            'object': fake_service()
        }
    ]
    return objs


def fake_service():
    return client.V1Service(
        api_version='v1',
        kind='Service',
        metadata=client.V1ObjectMeta(
            labels={
                'app': 'webserver',
                'vdu_name': 'curry-svc-vdu001'
            },
            name='curry-svc-vdu001',
            namespace='default'
        ),
        spec=client.V1ServiceSpec(
            cluster_ip=''
        )
    )


def fake_k8s_objs_service_false_cluster_ip():
    objs = [
        {
            'namespace': 'default',
            'status': 'Creating',
            'object': fake_service_false()
        }
    ]
    return objs


def fake_service_false():
    return client.V1Service(
        api_version='v1',
        kind='Service',
        metadata=client.V1ObjectMeta(
            labels={
                'app': 'webserver',
                'vdu_name': 'curry-svc-vdu001'
            },
            name='curry-svc-vdu001',
            namespace='default'
        ),
        spec=client.V1ServiceSpec(
            cluster_ip='127.0.0.1'
        )
    )


def fake_endpoinds():
    return client.V1Endpoints(
        api_version='v1',
        kind='Endpoints',
        metadata=client.V1ObjectMeta(
            namespace='default'
        )
    )


def fake_k8s_objs_deployment():
    obj = [
        {
            'namespace': 'test',
            'status': 'Creating',
            'object': fake_v1_deployment()
        }
    ]

    return obj


def fake_k8s_objs_deployment_error():
    obj = [
        {
            'namespace': 'test',
            'status': 'Creating',
            'object': fake_v1_deployment_error()
        }
    ]

    return obj


def fake_k8s_objs_replica_set():
    obj = [
        {
            'namespace': 'test',
            'status': 'Creating',
            'object': fake_v1_replica_set()
        }
    ]

    return obj


def fake_k8s_objs_replica_set_error():
    obj = [
        {
            'namespace': 'test',
            'status': 'Creating',
            'object': fake_v1_replica_set_error()
        }
    ]

    return obj


def fake_k8s_objs_stateful_set():
    obj = [
        {
            'namespace': 'test',
            'status': 'Creating',
            'object': fake_v1_stateful_set()
        }
    ]

    return obj


def fake_k8s_objs_stateful_set_error():
    obj = [
        {
            'namespace': 'test',
            'status': 'Creating',
            'object': fake_v1_stateful_set_error()
        }
    ]

    return obj


def fake_k8s_objs_job():
    obj = [
        {
            'namespace': 'test',
            'status': 'Creating',
            'object': fake_v1_job()
        }
    ]

    return obj


def fake_k8s_objs_job_error():
    obj = [
        {
            'namespace': 'test',
            'status': 'Creating',
            'object': fake_v1_job_error()
        }
    ]

    return obj


def fake_k8s_objs_volume_attachment():
    obj = [
        {
            'namespace': 'test',
            'status': 'Creating',
            'object': fake_v1_volume_attachment()
        }
    ]

    return obj


def fake_k8s_objs_volume_attachment_error():
    obj = [
        {
            'namespace': 'test',
            'status': 'Creating',
            'object': fake_v1_volume_attachment_error()
        }
    ]

    return obj


def fake_v1_deployment():
    return client.V1Deployment(
        api_version='apps/v1',
        kind='Deployment',
        metadata=client.V1ObjectMeta(
            name='curry-test001',
            namespace='curryns'
        ),
        status=client.V1DeploymentStatus(
            replicas=1,
            ready_replicas=1
        ),
        spec=client.V1DeploymentSpec(
            replicas=2,
            selector=client.V1LabelSelector(
                match_labels={'app': 'webserver'}
            ),
            template=client.V1PodTemplateSpec(
                metadata=client.V1ObjectMeta(
                    labels={'app': 'webserver'}
                )
            )
        )
    )


def fake_v1_deployment_error():
    return client.V1Deployment(
        api_version='apps/v1',
        kind='Deployment',
        metadata=client.V1ObjectMeta(
            name='curry-test001',
            namespace='curryns'
        ),
        status=client.V1DeploymentStatus(
            replicas=2,
            ready_replicas=1
        )
    )


def fake_v1_replica_set():
    return client.V1ReplicaSet(
        api_version='apps/v1',
        kind='ReplicaSet',
        metadata=client.V1ObjectMeta(
            name='curry-test001',
            namespace='curryns'
        ),
        status=client.V1ReplicaSetStatus(
            replicas=1,
            ready_replicas=1
        ),
        spec=client.V1ReplicaSetSpec(
            replicas=2,
            selector=client.V1LabelSelector(
                match_labels={'app': 'webserver'}
            ),
            template=client.V1PodTemplateSpec(
                metadata=client.V1ObjectMeta(
                    labels={'app': 'webserver'}
                )
            )
        )
    )


def fake_v1_replica_set_error():
    return client.V1ReplicaSet(
        api_version='apps/v1',
        kind='ReplicaSet',
        metadata=client.V1ObjectMeta(
            name='curry-test001',
            namespace='curryns'
        ),
        status=client.V1ReplicaSetStatus(
            replicas=2,
            ready_replicas=1
        )
    )


def fake_v1_job():
    return client.V1Job(
        api_version='batch/v1',
        kind='Job',
        metadata=client.V1ObjectMeta(
            name='curry-test001',
            namespace='curryns'
        ),
        spec=client.V1JobSpec(
            completions=1,
            template=client.V1PodTemplateSpec(
                metadata=client.V1ObjectMeta(
                    name='curry-test001',
                    namespace='curryns'
                ),
                spec=client.V1PodSpec(
                    hostname='job',
                    containers=['image']
                )
            )
        ),
        status=client.V1JobStatus(
            succeeded=1,
        )
    )


def fake_v1_job_error():
    return client.V1Job(
        api_version='batch/v1',
        kind='Job',
        metadata=client.V1ObjectMeta(
            name='curry-test001',
            namespace='curryns'
        ),
        spec=client.V1JobSpec(
            completions=1,
            template=client.V1PodTemplateSpec(
                metadata=client.V1ObjectMeta(
                    name='curry-test001',
                    namespace='curryns'
                ),
                spec=client.V1PodSpec(
                    hostname='job',
                    containers=['image']
                )
            )
        ),
        status=client.V1JobStatus(
            succeeded=2,
        )
    )


def fake_v1_volume_attachment():
    return client.V1VolumeAttachment(
        api_version='storage.k8s.io/v1',
        kind='VolumeAttachment',
        metadata=client.V1ObjectMeta(
            name='curry-test001',
            namespace='curryns'
        ),
        spec=client.V1VolumeAttachmentSpec(
            attacher='nginx',
            node_name='nginx',
            source=client.V1VolumeAttachmentSource(
                persistent_volume_name='curry-sc-pvc'
            )
        ),
        status=client.V1VolumeAttachmentStatus(
            attached=True,
        )
    )


def fake_v1_volume_attachment_error():
    return client.V1VolumeAttachment(
        api_version='storage.k8s.io/v1',
        kind='VolumeAttachment',
        metadata=client.V1ObjectMeta(
            name='curry-test001',
            namespace='curryns'
        ),
        spec=client.V1VolumeAttachmentSpec(
            attacher='nginx',
            node_name='nginx',
            source=client.V1VolumeAttachmentSource(
                persistent_volume_name='curry-sc-pvc'
            )
        ),
        status=client.V1VolumeAttachmentStatus(
            attached=False,
        )
    )


def fake_v1_stateful_set():
    return client.V1StatefulSet(
        api_version='apps/v1',
        kind='StatefulSet',
        metadata=client.V1ObjectMeta(
            name='curry-test001',
            namespace='curryns'
        ),
        spec=client.V1StatefulSetSpec(
            replicas=1,
            volume_claim_templates=[
                client.V1PersistentVolumeClaim(
                    metadata=client.V1ObjectMeta(
                        name='www'
                    )
                )
            ],
            selector=client.V1LabelSelector(
                match_labels={'app': 'nginx'}
            ),
            template=client.V1PodTemplateSpec(
                metadata=client.V1ObjectMeta(
                    name='curry-test001',
                    namespace='curryns'
                )
            ),
            service_name='nginx'
        ),
        status=client.V1StatefulSetStatus(
            replicas=1,
            ready_replicas=1
        ),
    )


def fake_v1_stateful_set_error():
    return client.V1StatefulSet(
        api_version='apps/v1',
        kind='StatefulSet',
        metadata=client.V1ObjectMeta(
            name='curry-test001',
            namespace='curryns'
        ),
        spec=client.V1StatefulSetSpec(
            replicas=1,
            volume_claim_templates=[
                client.V1PersistentVolumeClaim(
                    metadata=client.V1ObjectMeta(
                        name='www'
                    )
                )
            ],
            selector=client.V1LabelSelector(
                match_labels={'app': 'nginx'}
            ),
            template=client.V1PodTemplateSpec(
                metadata=client.V1ObjectMeta(
                    name='curry-test001',
                    namespace='curryns'
                )
            ),
            service_name='nginx'
        ),
        status=client.V1StatefulSetStatus(
            replicas=2,
            ready_replicas=1
        )
    )


def fake_v1_persistent_volume_claim():
    return client.V1PersistentVolumeClaim(
        api_version='v1',
        kind='PersistentVolumeClaim',
        metadata=client.V1ObjectMeta(
            name='www-curry-test001-0',
            namespace='curryns'
        ),
        status=client.V1PersistentVolumeClaimStatus(
            phase='Bound'
        )
    )


def fake_v1_persistent_volume_claim_error():
    return client.V1PersistentVolumeClaim(
        api_version='v1',
        kind='PersistentVolumeClaim',
        metadata=client.V1ObjectMeta(
            name='curry-test001',
            namespace='curryns'
        ),
        status=client.V1PersistentVolumeClaimStatus(
            phase='Bound1'
        )
    )


def fake_k8s_objs_pod():
    objs = [
        {
            'namespace': 'test',
            'status': 'Creating',
            'object': fake_pod()
        }
    ]
    return objs


def fake_k8s_objs_pod_error():
    objs = [
        {
            'namespace': 'test',
            'status': 'Creating',
            'object': fake_pod_error()
        }
    ]
    return objs


def fake_pod():
    return client.V1Pod(
        api_version='v1',
        kind='Pod',
        metadata=client.V1ObjectMeta(
            name='curry-test001',
            namespace='curryns'
        ),
        status=client.V1PodStatus(
            phase='Running',
        )
    )


def fake_pod_error():
    return client.V1Pod(
        api_version='v1',
        kind='Pod',
        metadata=client.V1ObjectMeta(
            name='curry-test001',
            namespace='curryns'
        ),
        status=client.V1PodStatus(
            phase='Terminated',
        )
    )


def fake_k8s_objs_persistent_volume():
    objs = [
        {
            'namespace': 'test',
            'status': 'Creating',
            'object': fake_persistent_volume()
        }
    ]
    return objs


def fake_k8s_objs_persistent_volume_error():
    objs = [
        {
            'namespace': 'test',
            'status': 'Creating',
            'object': fake_persistent_volume_error()
        }
    ]
    return objs


def fake_persistent_volume():
    return client.V1PersistentVolume(
        api_version='v1',
        kind='PersistentVolume',
        metadata=client.V1ObjectMeta(
            name='curry-test001',
            namespace='curryns'
        ),
        status=client.V1PersistentVolumeStatus(
            phase='Available',
        )
    )


def fake_persistent_volume_error():
    return client.V1PersistentVolume(
        api_version='v1',
        kind='PersistentVolume',
        metadata=client.V1ObjectMeta(
            name='curry-test001',
            namespace='curryns'
        ),
        status=client.V1PersistentVolumeStatus(
            phase='UnBound',
        )
    )


def fake_k8s_objs_api_service():
    objs = [
        {
            'namespace': 'test',
            'status': 'Creating',
            'object': fake_api_service()
        }
    ]
    return objs


def fake_k8s_objs_api_service_error():
    objs = [
        {
            'namespace': 'test',
            'status': 'Creating',
            'object': fake_api_service_error()
        }
    ]
    return objs


def fake_api_service():
    return client.V1APIService(
        api_version='apiregistration.k8s.io/v1',
        kind='APIService',
        metadata=client.V1ObjectMeta(
            name='curry-test001',
            namespace='curryns'
        ),
        status=client.V1APIServiceStatus(
            conditions=[
                client.V1APIServiceCondition(
                    type='Available',
                    status='True'
                )
            ]
        )
    )


def fake_api_service_error():
    return client.V1APIService(
        api_version='apiregistration.k8s.io/v1',
        kind='APIService',
        metadata=client.V1ObjectMeta(
            name='curry-test001',
            namespace='curryns'
        ),
        status=client.V1APIServiceStatus(
            conditions=[
                client.V1APIServiceCondition(
                    type='Unavailable',
                    status='True'
                )
            ]
        )
    )


def fake_k8s_objs_daemon_set():
    objs = [
        {
            'namespace': 'test',
            'status': 'Creating',
            'object': fake_daemon_set()
        }
    ]
    return objs


def fake_k8s_objs_daemon_set_error():
    objs = [
        {
            'namespace': 'test',
            'status': 'Creating',
            'object': fake_daemon_set_error()
        }
    ]
    return objs


def fake_daemon_set():
    return client.V1DaemonSet(
        api_version='apps/v1',
        kind='DaemonSet',
        metadata=client.V1ObjectMeta(
            name='curry-test001',
            namespace='curryns'
        ),
        status=client.V1DaemonSetStatus(
            number_ready=13,
            desired_number_scheduled=13,
            current_number_scheduled=4,
            number_misscheduled=2,
        ),
        spec=client.V1DaemonSetSpec(
            selector=client.V1LabelSelector(
                match_labels={'app': 'webserver'}
            ),
            template=client.V1PodTemplateSpec(
                metadata=client.V1ObjectMeta(
                    labels={'app': 'webserver'}
                )
            )
        )
    )


def fake_daemon_set_error():
    return client.V1DaemonSet(
        api_version='apps/v1',
        kind='DaemonSet',
        metadata=client.V1ObjectMeta(
            name='curry-test001',
            namespace='curryns'
        ),
        status=client.V1DaemonSetStatus(
            number_ready=13,
            desired_number_scheduled=12,
            current_number_scheduled=4,
            number_misscheduled=2,
        )
    )


def fake_auth_attr():
    return {
        'username': 'fake_user_name',
        'password': 'fake_password',
        'ssl_ca_cert': '-----BEGIN CERTIFICATE-----\
                        samplevalues\
                        -----END CERTIFICATE-----',
        'auth_url': 'http://fake-url/identity/v3'
    }


def fake_vnf_dict(updates=None):
    vnf_dict = {
        'id': 'fake-id,fake-name',
        'status': 'fake-status',
        'attributes': {
            'monitoring_policy': 'fake-monitoring-policy',
            'failure_count': '1',
            'dead_instance_id_1': '00000000-0000-0000-0000-00000000001'},
        'vim_id': 'fake-vim-id',
        'vim_auth': 'fake-vim-auth',
        'instance_id': '00000000-0000-0000-0000-000000000002',
        'placement_attr': {
            'region_name': 'fake-region-name'}}
    if updates:
        vnf_dict.update(updates)
    return vnf_dict


def fake_pod_list():
    return client.V1PodList(
        items=[client.V1Pod(
            metadata=client.V1ObjectMeta(
                name="fake-name"
            ),
            status=client.V1PodStatus(
                phase="Successed"
            )
        )]
    )


def get_scale_policy(type, aspect_id='vdu1', delta_num=1, is_legacy=False,
                     vdu_name='fake_name'):
    policy = dict()
    policy['action'] = type
    policy['name'] = aspect_id
    if is_legacy:
        policy['instance_id'] = "fake_namespace,fake_name"
    else:
        policy['vnf_instance_id'] = uuidsentinel.vnf_instance_id
        policy['delta_num'] = delta_num
        policy['vdu_defs'] = {
            'VDU1': {
                'type': 'tosca.nodes.nfv.Vdu.Compute',
                'properties': {
                    'name': vdu_name,
                    'description': 'test description',
                    'vdu_profile': {
                        'min_number_of_instances': 1,
                        'max_number_of_instances': 3}}}}

    return policy


def get_vnf_resource_list(kind, name='fake_name'):
    vnf_resource = models.VnfResource()
    vnf_resource.vnf_instance_id = uuidsentinel.vnf_instance_id
    vnf_resource.resource_name = name
    vnf_resource.resource_type = f"v1,{kind}"
    vnf_resource.tenant_id = uuidsentinel.tenant_id
    return [vnf_resource]


def get_fake_pod_info(kind, name='fake_name', pod_status='Running',
                      pod_name=None):
    if not pod_name:
        if kind == 'Deployment':
            pod_name = _('{name}-1234567890-abcde').format(name=name)
        elif kind == 'ReplicaSet' or kind == 'DaemonSet':
            pod_name = _('{name}-12345').format(name=name)
        elif kind == 'StatefulSet':
            pod_name = _('{name}-1').format(name=name)
        elif kind == 'Pod':
            pod_name = name
    return client.V1Pod(
        metadata=client.V1ObjectMeta(name=pod_name,
            creation_timestamp=datetime.datetime.now().isoformat('T')),
        status=client.V1PodStatus(phase=pod_status))


def fake_vnfc_resource_info(vdu_id='VDU1', rsc_kind='Deployment',
                            rsc_name='fake_name', pod_name=None,
                            namespace=None):
    def _get_metadata_str(name, namespace="fake_namespace"):
        if namespace == "brank":
            namespace = ""
        metadata = {
            'name': name,
            'namespace': namespace}
        return jsonutils.dumps(metadata)

    vnfc_obj = objects.VnfcResourceInfo()
    vnfc_obj.id = uuidutils.generate_uuid()
    vnfc_obj.vdu_id = vdu_id
    if not pod_name:
        v1_pod = get_fake_pod_info(rsc_kind, rsc_name)
        pod_name = v1_pod.metadata.name
    compute_resource = objects.ResourceHandle(
        resource_id=pod_name,
        vim_level_resource_type=rsc_kind)
    vnfc_obj.compute_resource = compute_resource
    metadata = {}
    if namespace:
        metadata['Pod'] = _get_metadata_str(
            name=pod_name, namespace=namespace)
        if rsc_kind != 'Pod':
            metadata[rsc_kind] = _get_metadata_str(
                name=rsc_name, namespace=namespace)
    else:
        metadata['Pod'] = _get_metadata_str(name=pod_name)
        if rsc_kind != 'Pod':
            metadata[rsc_kind] = _get_metadata_str(name=rsc_name)
    vnfc_obj.metadata = metadata

    return vnfc_obj


def fake_vim_connection_info():
    access_info = {
        'auth_url': 'http://fake_url:6443',
        'ssl_ca_cert': None}

    return vim_connection.VimConnectionInfo(
        vim_type="kubernetes",
        access_info=access_info)


def fake_vim_connection_info_with_extra(del_field=None, multi_ip=False):
    access_info = {
        'auth_url': 'http://fake_url:6443',
        'ssl_ca_cert': None}
    masternode_ip = ["192.168.0.1"]
    if multi_ip:
        masternode_ip.append("192.168.0.2")

    helm_info = {
        'masternode_ip': masternode_ip,
        'masternode_username': 'dummy_user',
        'masternode_password': 'dummy_pass'
    }
    if del_field and helm_info.get(del_field):
        del helm_info[del_field]
    extra = {
        'helm_info': str(helm_info)
    }
    return vim_connection.VimConnectionInfo(
        vim_type="kubernetes",
        access_info=access_info,
        extra=extra)


def fake_inst_vnf_req_for_helmchart(external=True, local=True, namespace=None):
    additional_params = {"use_helm": "true"}
    using_helm_install_param = list()
    if external:
        using_helm_install_param.append(
            {
                "exthelmchart": "true",
                "helmreleasename": "myrelease-ext",
                "helmrepositoryname": "sample-charts",
                "helmchartname": "mychart-ext",
                "exthelmrepo_url": "http://helmrepo.example.com/sample-charts"
            }
        )
    if local:
        using_helm_install_param.append(
            {
                "exthelmchart": "false",
                "helmchartfile_path": "Files/kubernetes/localhelm-0.1.0.tgz",
                "helmreleasename": "myrelease-local",
                "helmparameter": [
                    "key1=value1",
                    "key2=value2"
                ]
            }
        )
    additional_params['using_helm_install_param'] = using_helm_install_param
    additional_params['helm_replica_values'] = {"vdu1_aspect": "replicaCount"}
    if namespace:
        additional_params['namespace'] = namespace

    return objects.InstantiateVnfRequest(
        flavour_id="simple", additional_params=additional_params)


def execute_cmd_helm_client(*args, **kwargs):
    ssh_command = args[0]
    if 'helm get manifest' in ssh_command:
        result = [
            '---\n',
            '# Source: localhelm/templates/deployment.yaml\n',
            'apiVersion: apps/v1\n',
            'kind: Deployment\n',
            'metadata:\n',
            '  name: vdu1\n',
            'spec:\n',
            '  replicas: 1\n',
            '  selector:\n',
            '    matchLabels:\n',
            '      app: webserver\n',
            '  template:\n',
            '    metadata:\n'
            '      labels:\n'
            '        app: webserver\n'
            '    spec:\n',
            '      containers:\n',
            '        - name: nginx\n'
        ]
    elif 'helm get values' in ssh_command:
        result = ['{"replicaCount":2}']
    else:
        result = ""
    return result


def fake_k8s_objs_deployment_for_helm():
    obj = [
        {
            'status': 'Creating',
            'object': fake_v1_deployment_for_helm()
        }
    ]

    return obj


def fake_v1_deployment_for_helm():
    return client.V1Deployment(
        api_version='apps/v1',
        kind='Deployment',
        metadata=client.V1ObjectMeta(
            name='vdu1',
        ),
        status=client.V1DeploymentStatus(
            replicas=1,
            ready_replicas=1
        ),
        spec=client.V1DeploymentSpec(
            replicas=1,
            selector=client.V1LabelSelector(
                match_labels={'app': 'webserver'}
            ),
            template=client.V1PodTemplateSpec(
                metadata=client.V1ObjectMeta(
                    labels={'app': 'webserver'}
                ),
                spec=client.V1PodSpec(
                    containers=[
                        client.V1Container(
                            name='nginx'
                        )
                    ]
                )
            )
        )
    )


def fake_k8s_vim_obj():
    vim_obj = {'vim_id': '76107920-e588-4865-8eca-f33a0f827071',
               'vim_name': 'fake_k8s_vim',
               'vim_auth': {
                   'auth_url': 'http://localhost:6443',
                   'password': 'test_pw',
                   'username': 'test_user',
                   'project_name': 'test_project'},
               'vim_type': 'kubernetes',
               'tenant': uuidsentinel.tenant_id,
               'extra': {}}
    return vim_obj



# Copyright (C) 2020 NTT DATA
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_utils import timeutils
from oslo_utils import uuidutils
from oslo_versionedobjects import base as ovoo_base

from tacker._i18n import _
from tacker.common import exceptions
from tacker.db import api as db_api
from tacker.db.db_sqlalchemy import api
from tacker.db.db_sqlalchemy import models
from tacker.objects import base
from tacker.objects import fields


@db_api.context_manager.writer
def _vnf_resource_create(context, values):
    vnf_resource = models.VnfResource()
    vnf_resource.update(values)
    vnf_resource.save(context.session)

    return vnf_resource


@db_api.context_manager.reader
def _vnf_resource_get_by_id(context, id):

    query = api.model_query(context, models.VnfResource,
                            read_deleted="no", project_only=True). \
        filter_by(id=id)

    result = query.first()

    if not result:
        raise exceptions.VnfResourceNotFound(id=id)

    return result


@db_api.context_manager.writer
def _vnf_resource_update(context, id, values):

    vnf_resource = _vnf_resource_get_by_id(context, id)
    vnf_resource.update(values)
    vnf_resource.save(session=context.session)

    return vnf_resource


@db_api.context_manager.writer
def _destroy_vnf_resource(context, id):
    now = timeutils.utcnow()
    updated_values = {'deleted': True,
                      'deleted_at': now
                      }

    api.model_query(context, models.VnfResource).\
        filter_by(id=id). \
        update(updated_values, synchronize_session=False)


@db_api.context_manager.reader
def _vnf_resource_list(context, vnf_instance_id):
    query = api.model_query(context, models.VnfResource, read_deleted="no",
                            project_only=True).\
        filter_by(vnf_instance_id=vnf_instance_id)

    return query.all()


def _make_vnf_resources_list(context, vnf_resource_list, db_vnf_resource_list):
    vnf_resource_cls = VnfResource

    vnf_resource_list.objects = []
    for db_vnf_resource in db_vnf_resource_list:
        vnf_resource_obj = vnf_resource_cls._from_db_object(
            context, vnf_resource_cls(context), db_vnf_resource)
        vnf_resource_list.objects.append(vnf_resource_obj)

    vnf_resource_list.obj_reset_changes()
    return vnf_resource_list


@base.TackerObjectRegistry.register
class VnfResource(base.TackerObject, base.TackerPersistentObject):

    # Version 1.0: Initial version
    VERSION = '1.0'

    fields = {
        'id': fields.UUIDField(nullable=False),
        'vnf_instance_id': fields.StringField(nullable=False),
        'resource_name': fields.StringField(nullable=True),
        'resource_type': fields.StringField(nullable=False),
        'resource_identifier': fields.StringField(nullable=False),
        'resource_status': fields.StringField(nullable=True, default='status'),
        'tenant_id': fields.StringField(nullable=False)
    }

    def __init__(self, context=None, **kwargs):
        super(VnfResource, self).__init__(context, **kwargs)
        self.obj_set_defaults()

    @staticmethod
    def _from_db_object(context, vnf_resource, db_vnf_resource):

        for key in vnf_resource.fields:
            if db_vnf_resource[key]:
                setattr(vnf_resource, key, db_vnf_resource[key])

        vnf_resource._context = context
        vnf_resource.obj_reset_changes()

        return vnf_resource

    @base.remotable
    def create(self):
        if self.obj_attr_is_set('id'):
            raise exceptions.ObjectActionError(action='create',
                                               reason=_('already created'))
        updates = self.obj_get_changes()

        if 'id' not in updates:
            updates['id'] = uuidutils.generate_uuid()
            self.id = updates['id']

        db_vnf_resource = _vnf_resource_create(self._context, updates)
        self._from_db_object(self._context, self, db_vnf_resource)

    @base.remotable
    def save(self):
        updates = self.tacker_obj_get_changes()

        db_vnf_resource = _vnf_resource_update(self._context,
                                            self.id, updates)
        self._from_db_object(self._context, self, db_vnf_resource)

    @base.remotable
    def destroy(self, context):
        if not self.obj_attr_is_set('id'):
            raise exceptions.ObjectActionError(action='destroy',
                                               reason='no uuid')

        _destroy_vnf_resource(context, self.id)

    @base.remotable_classmethod
    def get_by_id(cls, context, id):
        db_vnf_package = _vnf_resource_get_by_id(context, id)
        return cls._from_db_object(context, cls(), db_vnf_package)


@base.TackerObjectRegistry.register
class VnfResourceList(ovoo_base.ObjectListBase, base.TackerObject):

    VERSION = '1.0'

    fields = {
        'objects': fields.ListOfObjectsField('VnfResource')
    }

    @base.remotable_classmethod
    def get_by_vnf_instance_id(cls, context, vnf_instance_id):
        db_vnf_resources = _vnf_resource_list(context, vnf_instance_id)
        return _make_vnf_resources_list(context, cls(), db_vnf_resources)



# Copyright 2015 Brocade Communications System, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


from tacker.plugins.common import constants as evt_constants
from tacker.tests import constants
from tacker.tests.functional import base


class VnfTestPingMonitor(base.BaseTackerTest):

    def _test_vnf_with_monitoring(self, vnfd_file, vnf_name):
        vnfd_instance, vnf_instance, tosca_dict = self.vnfd_and_vnf_create(
            vnfd_file, vnf_name)

        # Verify vnf goes from ACTIVE->DEAD->ACTIVE states
        self.verify_vnf_restart(vnfd_instance, vnf_instance)

        # Delete vnf_instance with vnf_id
        vnf_id = vnf_instance['vnf']['id']
        try:
            self.client.delete_vnf(vnf_id)
        except Exception:
            assert False, ("Failed to delete vnf %s after the monitor test" %
                           vnf_id)

        # Verify VNF monitor events captured for states, ACTIVE and DEAD
        vnf_state_list = [evt_constants.ACTIVE, evt_constants.DEAD]
        self.verify_vnf_monitor_events(vnf_id, vnf_state_list)

        self.addCleanup(self.wait_until_vnf_delete, vnf_id,
            constants.VNF_CIRROS_DELETE_TIMEOUT)

    def test_create_delete_vnf_monitoring_tosca_template(self):
        self._test_vnf_with_monitoring(
            'sample-tosca-vnfd-monitor.yaml',
            'ping monitor vnf with tosca template')

    def test_create_delete_vnf_multi_vdu_monitoring_tosca_template(self):
        self._test_vnf_with_monitoring(
            'sample-tosca-vnfd-multi-vdu-monitoring.yaml',
            'ping monitor multi vdu vnf with tosca template')

    def _test_vnf_with_monitoring_vdu_autoheal_action(
            self, vnfd_file, vnf_name):
        vnfd_instance, vnf_instance, tosca_dict = self.vnfd_and_vnf_create(
            vnfd_file, vnf_name)
        vnf_id = vnf_instance['vnf']['id']

        self.verify_vnf_update(vnf_id)

        # Delete vnf_instance with vnf_id
        try:
            self.client.delete_vnf(vnf_id)
        except Exception:
            assert False, ("Failed to delete vnf %s after the monitor test" %
                           vnf_id)
        self.addCleanup(self.wait_until_vnf_delete, vnf_id,
                        constants.VNF_CIRROS_DELETE_TIMEOUT)

        params = {'resource_id': vnf_id,
                  'resource_state': 'PENDING_UPDATE',
                  'event_type': evt_constants.RES_EVT_MONITOR}
        vnf_events = self.client.list_vnf_events(**params)
        # Check if vdu_autoheal action emits 4 monitoring events.
        self.assertGreaterEqual(4, len(vnf_events['vnf_events']))

    def test_vnf_monitoring_with_vdu_autoheal_action_for_multi_vdu(self):
        self._test_vnf_with_monitoring_vdu_autoheal_action(
            'sample-tosca-vnfd-multi-vdu-monitoring-vdu-autoheal.yaml',
            'ping multi vdu monitor having vdu_autoheal failure action '
            'with tosca template')

    def test_vnf_monitoring_with_vdu_autoheal_action_for_single_vdu(self):
        self._test_vnf_with_monitoring_vdu_autoheal_action(
            'sample-tosca-vnfd-single-vdu-monitoring-vdu-autoheal.yaml',
            'ping vdu monitor having vdu_autoheal failure action '
            'with tosca template')



# Copyright 2012 VMware, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from unittest import mock

from oslo_context import context as oslo_context
from testtools import matchers

from tacker import context
from tacker.tests import base


class TestTackerContext(base.BaseTestCase):

    def setUp(self):
        super(TestTackerContext, self).setUp()
        db_api = 'tacker.db.api.get_session'
        self._db_api_session_patcher = mock.patch(db_api)
        self.db_api_session = self._db_api_session_patcher.start()

    def test_tacker_context_create(self):
        ctx = context.Context('user_id', 'tenant_id')
        self.assertEqual('user_id', ctx.user_id)
        self.assertEqual('tenant_id', ctx.project_id)
        self.assertEqual('tenant_id', ctx.tenant_id)
        self.assertThat(ctx.request_id, matchers.StartsWith('req-'))
        self.assertEqual('user_id', ctx.user_id)
        self.assertEqual('tenant_id', ctx.project_id)
        self.assertIsNone(ctx.user_name)
        self.assertIsNone(ctx.tenant_name)

    def test_tacker_context_create_with_name(self):
        ctx = context.Context('user_id', 'tenant_id',
                              tenant_name='tenant_name', user_name='user_name')
        # Check name is set
        self.assertEqual('user_name', ctx.user_name)
        self.assertEqual('tenant_name', ctx.tenant_name)
        # Check user/tenant contains its ID even if user/tenant_name is passed
        self.assertEqual('user_id', ctx.user_id)
        self.assertEqual('tenant_id', ctx.project_id)

    def test_tacker_context_create_with_request_id(self):
        ctx = context.Context('user_id', 'tenant_id', request_id='req_id_xxx')
        self.assertEqual('req_id_xxx', ctx.request_id)

    def test_tacker_context_to_dict(self):
        ctx = context.Context('user_id', 'tenant_id')
        ctx_dict = ctx.to_dict()
        self.assertEqual('user_id', ctx_dict['user_id'])
        self.assertEqual('tenant_id', ctx_dict['project_id'])
        self.assertEqual(ctx.request_id, ctx_dict['request_id'])
        self.assertEqual('user_id', ctx_dict['user_id'])
        self.assertEqual('tenant_id', ctx_dict['project_id'])
        self.assertIsNone(ctx_dict['user_name'])
        self.assertIsNone(ctx_dict['tenant_name'])
        self.assertIsNone(ctx_dict['project_name'])

    def test_tacker_context_to_dict_with_name(self):
        ctx = context.Context('user_id', 'tenant_id',
                              tenant_name='tenant_name', user_name='user_name')
        ctx_dict = ctx.to_dict()
        self.assertEqual('user_name', ctx_dict['user_name'])
        self.assertEqual('tenant_name', ctx_dict['tenant_name'])
        self.assertEqual('tenant_name', ctx_dict['project_name'])

    def test_tacker_context_admin_to_dict(self):
        self.db_api_session.return_value = 'fakesession'
        ctx = context.get_admin_context()
        ctx_dict = ctx.to_dict()
        self.assertIsNone(ctx_dict['user_id'])
        self.assertIsNone(ctx_dict['tenant_id'])
        self.assertIsNotNone(ctx.session)
        self.assertNotIn('session', ctx_dict)

    def test_tacker_context_admin_without_session_to_dict(self):
        ctx = context.get_admin_context_without_session()
        ctx_dict = ctx.to_dict()
        self.assertIsNone(ctx_dict['user_id'])
        self.assertIsNone(ctx_dict['tenant_id'])
        self.assertFalse(hasattr(ctx, 'session'))

    def test_tacker_context_admin_context(self):
        ctx = context.get_admin_context()
        self.assertTrue(ctx.is_admin)
        self.assertFalse(ctx.roles)

    def test_tacker_context_elevated_retains_request_id(self):
        ctx = context.Context('user_id', 'tenant_id')
        self.assertFalse(ctx.is_admin)
        req_id_before = ctx.request_id

        elevated_ctx = ctx.elevated()
        self.assertTrue(elevated_ctx.is_admin)
        self.assertEqual(req_id_before, elevated_ctx.request_id)

    def test_tacker_context_overwrite(self):
        ctx1 = context.Context('user_id', 'tenant_id')
        self.assertEqual(oslo_context.get_current().request_id,
                         ctx1.request_id)

        # If overwrite is not specified, request_id should be updated.
        ctx2 = context.Context('user_id', 'tenant_id')
        self.assertNotEqual(ctx2.request_id, ctx1.request_id)
        self.assertEqual(oslo_context.get_current().request_id,
                         ctx2.request_id)

        # If overwrite is specified, request_id should be kept.
        ctx3 = context.Context('user_id', 'tenant_id', overwrite=False)
        self.assertNotEqual(ctx3.request_id, ctx2.request_id)
        self.assertEqual(oslo_context.get_current().request_id,
                         ctx2.request_id)

    def test_tacker_context_get_admin_context_not_update_local_store(self):
        ctx = context.Context('user_id', 'tenant_id')
        req_id_before = oslo_context.get_current().request_id
        self.assertEqual(req_id_before, ctx.request_id)

        ctx_admin = context.get_admin_context()
        self.assertEqual(req_id_before,
                         oslo_context.get_current().request_id)
        self.assertNotEqual(req_id_before, ctx_admin.request_id)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tacker.common import exceptions
from tacker import objects
from tacker.tests.unit import base
from tacker.tests.unit.vnfm.infra_drivers.openstack.fixture_data import (
    fixture_data_utils as fd_utils
)
from tacker.vnfm.infra_drivers.kubernetes import utils as k8s_utils
from unittest import mock


class KubernetesUtilsTestCase(base.TestCase):

    @mock.patch('tacker.objects.vnf_instance.VnfInstance.save')
    def test_check_and_save_namespace_multi_namespace(self, mock_save):
        instantiate_vnf_req = objects.InstantiateVnfRequest(
            additional_params=None)
        chk_namespaces = [{"namespace": "a", "kind": "CronJob"},
                          {"namespace": "b", "kind": "Deployment"},
                          {"namespace": "c", "kind": "Service"}]
        vnf_instance = fd_utils.get_vnf_instance_object()

        self.assertRaises(
            exceptions.NamespaceIsNotUnique,
            k8s_utils.check_and_save_namespace, instantiate_vnf_req,
            chk_namespaces, vnf_instance)

    @mock.patch('tacker.objects.vnf_instance.VnfInstance.save')
    def test_check_and_save_namespace_no_namespace(self, mock_save):
        instantiate_vnf_req = objects.InstantiateVnfRequest(
            additional_params=None)
        chk_namespaces = []
        vnf_instance = fd_utils.get_vnf_instance_object()
        vnf_instance.vnf_metadata['namespace'] = ''
        k8s_utils.check_and_save_namespace(
            instantiate_vnf_req, chk_namespaces, vnf_instance)
        self.assertEqual('default', vnf_instance.vnf_metadata['namespace'])

    @mock.patch('tacker.objects.vnf_instance.VnfInstance.save')
    def test_check_and_save_namespace_additional_params(self, mock_save):
        instantiate_vnf_req = objects.InstantiateVnfRequest(
            additional_params={'namespace': 'ns1'})
        chk_namespaces = []
        vnf_instance = fd_utils.get_vnf_instance_object()
        vnf_instance.vnf_metadata['namespace'] = ''
        k8s_utils.check_and_save_namespace(
            instantiate_vnf_req, chk_namespaces, vnf_instance)
        self.assertEqual('ns1', vnf_instance.vnf_metadata['namespace'])

    @mock.patch('tacker.objects.vnf_instance.VnfInstance.save')
    def test_check_and_save_namespace_manifests(self, mock_save):
        instantiate_vnf_req = objects.InstantiateVnfRequest(
            additional_params=None)
        chk_namespaces = [{"namespace": "ns2", "kind": "Deployment"}]
        vnf_instance = fd_utils.get_vnf_instance_object()
        vnf_instance.vnf_metadata = {}
        k8s_utils.check_and_save_namespace(
            instantiate_vnf_req, chk_namespaces, vnf_instance)
        self.assertEqual('ns2', vnf_instance.vnf_metadata['namespace'])



# Copyright 2013, 2014 Intel Corporation.
# All Rights Reserved.
#
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import abc

from tacker.api import extensions


class VnfAbstractDriver(extensions.PluginInterface, metaclass=abc.ABCMeta):

    @abc.abstractmethod
    def get_type(self):
        """Return one of predefined type of the hosting vnf drivers."""
        pass

    @abc.abstractmethod
    def get_name(self):
        """Return a symbolic name for the service VM plugin."""
        pass

    @abc.abstractmethod
    def get_description(self):
        pass

    @abc.abstractmethod
    def create(self, plugin, context, vnf):
        """Create vnf and return its id."""

    @abc.abstractmethod
    def create_wait(self, plugin, context, vnf_dict, vnf_id):
        """wait for vnf creation to complete."""

    @abc.abstractmethod
    def update(self, plugin, context, vnf_id, vnf_dict, vnf):
        # vnf_dict: old vnf_dict to be updated
        # vnf: update with vnf dict
        pass

    @abc.abstractmethod
    def update_wait(self, plugin, context, vnf_dict):
        pass

    @abc.abstractmethod
    def delete(self, plugin, context, vnf_id):
        pass

    @abc.abstractmethod
    def delete_wait(self, plugin, context, vnf_id):
        pass

    @abc.abstractmethod
    def get_resource_info(self, plugin, context, vnf_info, auth_attr,
                          region_name=None):
        '''Fetches optional details of a VNF'''
        pass

    @abc.abstractmethod
    def heal_vdu(self, plugin, context, vnf_dict, heal_request_data):
        pass

    @abc.abstractmethod
    def pre_instantiation_vnf(self, context, vnf_instance,
                              vim_connection_info, vnf_software_images):
        """Create resources required for instantiating Vnf.

        :param context: A RequestContext
        :param vnf_instance: Object tacker.objects.VnfInstance
        :vim_info: Credentials to initialize Vim connection
        :vnf_software_images: Dict of key:value pair,
          <VDU/Storage node name>:tacker.objects.VnfSoftwareImage.
        """
        pass

    @abc.abstractmethod
    def delete_vnf_instance_resource(self, context, vnf_instance,
            vim_connection_info, vnf_resource):
        pass

    @abc.abstractmethod
    def instantiate_vnf(self, context, vnf_instance, vnfd_dict,
                        vim_connection_info, instantiate_vnf_req,
                        grant_response):
        pass

    @abc.abstractmethod
    def post_vnf_instantiation(self, context, vnf_instance,
                               vim_connection_info, instantiate_vnf_req):
        pass

    @abc.abstractmethod
    def heal_vnf(self, context, vnf_instance, vim_connection_info,
                 heal_vnf_request):
        """Heal vnf

        :param context: A RequestContext
        :param vnf_instance: tacker.objects.VnfInstance to be healed
        :vim_info: Credentials to initialize Vim connection
        :heal_vnf_request: tacker.objects.HealVnfRequest object containing
                           parameters passed in the heal request
        """
        pass

    @abc.abstractmethod
    def heal_vnf_wait(self, context, vnf_instance, vim_connection_info,
                      heal_vnf_request):
        """Check vnf is healed successfully"""
        pass

    @abc.abstractmethod
    def post_heal_vnf(self, context, vnf_instance, vim_connection_info,
                      heal_vnf_request):
        """Update resource_id for each vnfc resources

        :param context: A RequestContext
        :param vnf_instance: tacker.objects.VnfInstance to be healed
        :vim_info: Credentials to initialize Vim connection
        :heal_vnf_request: tacker.objects.HealVnfRequest object containing
                           parameters passed in the heal request
        """
        pass

    @abc.abstractmethod
    def change_ext_conn_vnf(self, context, vnf_instance, vnf_dict,
                            vim_connection_info, change_ext_conn_req):
        """Change external VNF connectivity

        :param context: A RequestContext
        :param vnf_instance: tacker.objects.VnfInstance to be changed
        :param vnf_dict:
        :param vim_connection_info: Credentials to initialize Vim connection
        :param change_ext_conn_req: tacker.objects.ChangeExtconnRequest object
                                    containing parameters passed in the
                                    change_ext_conn request
        """
        pass

    @abc.abstractmethod
    def change_ext_conn_vnf_wait(self, context, vnf_instance,
                                 vim_connection_info):
        """Check vnf external connnectivity is changed successfully"""
        pass

    @abc.abstractmethod
    def post_change_ext_conn_vnf(self, context, vnf_instance,
                                 vim_connection_info):
        """Update resource information for each external VL/LINKPORT resources

        :param context: A RequestContext
        :param vnf_instance: tacker.objects.VnfInstance to be changed
        :param vim_connection_info: Credentials to initialize Vim connection
        """
        pass




# Copyright 2015-2016 Brocade Communications Systems Ine All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import os

from cryptography import fernet
from oslo_config import cfg
from oslo_log import log as logging


from tacker.common import utils
from tacker import context as t_context
from tacker.extensions import nfvo
from tacker.keymgr import API as KEYMGR_API
from tacker import manager
from tacker.plugins.common import constants

LOG = logging.getLogger(__name__)
CONF = cfg.CONF


class VimClient(object):
    def get_vim(self, context, vim_id=None, region_name=None):
        """Get Vim information for provided VIM id

        Initiate the NFVO plugin, request VIM information for the provided
        VIM id and validate region
        """
        nfvo_plugin = manager.TackerManager.get_service_plugins().get(
            constants.NFVO)

        if not vim_id:
            LOG.debug('VIM id not provided. Attempting to find default '
                      'VIM information')
            try:
                vim_info = nfvo_plugin.get_default_vim(context)
            except Exception as ex:
                LOG.debug('Fail to get default vim due to %s', ex)
                raise nfvo.VimDefaultNotDefined()
        else:
            try:
                vim_info = nfvo_plugin.get_vim(context, vim_id,
                                               mask_password=False)
            except Exception:
                raise nfvo.VimNotFoundException(vim_id=vim_id)
        LOG.debug('VIM info found for vim id %s', vim_id)
        if region_name and not self.region_valid(vim_info['placement_attr']
                                                 ['regions'], region_name):
            raise nfvo.VimRegionNotFoundException(region_name=region_name)

        vim_auth = self._build_vim_auth(vim_info)
        vim_res = {'vim_auth': vim_auth, 'vim_id': vim_info['id'],
                   'vim_name': vim_info.get('name', vim_info['id']),
                   'vim_type': vim_info['type'],
                   'tenant': vim_info['tenant_id'],
                   'placement_attr': vim_info.get('placement_attr', {}),
                   'extra': vim_info.get('extra', {})}
        return vim_res

    @staticmethod
    def region_valid(vim_regions, region_name):
        return region_name in vim_regions

    def _build_vim_auth(self, vim_info):
        LOG.debug('VIM id is %s', vim_info['id'])
        vim_auth = vim_info['auth_cred']

        # decode password
        if ('password' in vim_auth) and (vim_auth['password'] is not None):
            vim_auth['password'] = self._decode_vim_auth(vim_info['id'],
                                                         vim_auth,
                                                         vim_auth['password'])
        # decode bearer_token
        if 'bearer_token' in vim_auth:
            vim_auth['bearer_token'] = self.\
                _decode_vim_auth(vim_info['id'],
                                 vim_auth,
                                 vim_auth['bearer_token'])
        # decode ssl_ca_cert
        if utils.none_from_string(vim_auth.get('ssl_ca_cert')) is not None:
            vim_auth['ssl_ca_cert'] = self.\
                _decode_vim_auth(vim_info['id'],
                                 vim_auth,
                                 vim_auth['ssl_ca_cert'])

        vim_auth['auth_url'] = vim_info['auth_url']

        # These attributes are needless for authentication
        # from keystone, so we remove them.
        needless_attrs = ['key_type', 'secret_uuid']
        for attr in needless_attrs:
            if attr in vim_auth:
                vim_auth.pop(attr, None)
        return vim_auth

    def _decode_vim_auth(self, vim_id, auth, secret_value):
        """Decode Vim credentials

        Decrypt VIM cred, get fernet Key from local_file_system or
        barbican.
        """
        cred = secret_value.encode('utf-8')
        if auth.get('key_type') == 'barbican_key':
            keystone_conf = CONF.keystone_authtoken
            secret_uuid = auth['secret_uuid']
            keymgr_api = KEYMGR_API(keystone_conf.auth_url)
            k_context = t_context.generate_tacker_service_context()
            secret_obj = keymgr_api.get(k_context, secret_uuid)
            vim_key = secret_obj.payload
        else:
            vim_key = self._find_vim_key(vim_id)

        f = fernet.Fernet(vim_key)
        if not f:
            LOG.warning('Unable to decode VIM auth')
            raise nfvo.VimNotFoundException(vim_id=vim_id)
        return f.decrypt(cred).decode('utf-8')

    @staticmethod
    def _find_vim_key(vim_id):
        key_file = os.path.join(CONF.vim_keys.openstack, vim_id)
        LOG.debug('Attempting to open key file for vim id %s', vim_id)
        try:
            with open(key_file, 'r') as f:
                return f.read()
        except Exception:
            LOG.warning('VIM id invalid or key not found for  %s', vim_id)
            raise nfvo.VimKeyNotFoundException(vim_id=vim_id)



# Copyright (C) 2021 Nippon Telegraph and Telephone Corporation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import os
import pickle
import sys


class SampleScript(object):

    def __init__(self, req, inst, grant_req, grant, csar_dir):
        self.req = req
        self.inst = inst
        self.grant_req = grant_req
        self.grant = grant
        self.csar_dir = csar_dir

    def instantiate_start(self):
        pass

    def instantiate_end(self):
        pass

    def terminate_start(self):
        pass

    def terminate_end(self):
        pass


def main():
    script_dict = pickle.load(sys.stdin.buffer)

    operation = script_dict['operation']
    req = script_dict['request']
    inst = script_dict['vnf_instance']
    grant_req = script_dict['grant_request']
    grant = script_dict['grant_response']
    csar_dir = script_dict['tmp_csar_dir']

    script = SampleScript(req, inst, grant_req, grant, csar_dir)
    try:
        getattr(script, operation)()
    except AttributeError:
        raise Exception("{} is not included in the script.".format(operation))


if __name__ == "__main__":
    try:
        main()
        os._exit(0)
    except Exception as ex:
        sys.stderr.write(str(ex))
        sys.stderr.flush()
        os._exit(1)



# Copyright 2015 Intel Corporation.
# All Rights Reserved.
#
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_config import cfg
from oslo_log import log as logging
from oslo_serialization import jsonutils
import yaml

from tacker._i18n import _
from tacker.common import cmd_executer
from tacker.common import exceptions
from tacker.common import log
from tacker.vnfm.mgmt_drivers import abstract_driver
from tacker.vnfm.mgmt_drivers import constants as mgmt_constants


LOG = logging.getLogger(__name__)
OPTS = [
    cfg.StrOpt('user', default='root', help=_('User name to login openwrt')),
    cfg.StrOpt('password', default='', help=_('Password to login openwrt')),
]
cfg.CONF.register_opts(OPTS, 'openwrt')


def config_opts():
    return [('openwrt', OPTS)]


class VnfMgmtOpenWRT(abstract_driver.VnfMGMTAbstractDriver):
    def get_type(self):
        return 'openwrt'

    def get_name(self):
        return 'openwrt'

    def get_description(self):
        return 'Tacker VNFMgmt OpenWRT Driver'

    def mgmt_ip_address(self, plugin, context, vnf):
        LOG.debug('mgmt_ip_address %s', vnf)
        return vnf.get('mgmt_ip_address', '')

    @log.log
    def _config_service(self, mgmt_ip_address, service, config):
        user = cfg.CONF.openwrt.user
        password = cfg.CONF.openwrt.password
        package = service
        if service == "dhcp":
            package = "dnsmasq"
        try:
            cmd = "uci import %s; /etc/init.d/%s restart" % (service, package)
            LOG.debug('execute command: %(cmd)s on mgmt_ip_address '
                      '%(mgmt_ip)s',
                      {'cmd': cmd,
                       'mgmt_ip': mgmt_ip_address})
            commander = cmd_executer.RemoteCommandExecutor(
                user, password, mgmt_ip_address)
            commander.execute_command(cmd, input_data=config)
        except Exception as ex:
            LOG.error("While executing command on remote "
                      "%(mgmt_ip)s: %(exception)s",
                      {'mgmt_ip': mgmt_ip_address,
                       'exception': ex})
            raise exceptions.MgmtDriverException()

    @log.log
    def mgmt_call(self, plugin, context, vnf, kwargs):
        if (kwargs[mgmt_constants.KEY_ACTION] !=
                mgmt_constants.ACTION_UPDATE_VNF):
            return
        dev_attrs = vnf.get('attributes', {})

        mgmt_ip_address = jsonutils.loads(vnf.get('mgmt_ip_address', '{}'))
        if not mgmt_ip_address:
            return

        vdus_config = dev_attrs.get('config', '')
        config_yaml = yaml.safe_load(vdus_config)
        if not config_yaml:
            return
        vdus_config_dict = config_yaml.get('vdus', {})
        for vdu, vdu_dict in vdus_config_dict.items():
            config = vdu_dict.get('config', {})
            for key, conf_value in config.items():
                KNOWN_SERVICES = ('firewall', 'network', 'dhcp', 'qos')
                if key not in KNOWN_SERVICES:
                    continue
                mgmt_ip_address = mgmt_ip_address.get(vdu, '')
                if not mgmt_ip_address:
                    LOG.warning('tried to configure unknown mgmt '
                                'address on VNF %(vnf)s VDU %(vdu)s',
                                {'vnf': vnf.get('name'),
                                 'vdu': vdu})
                    continue

                if isinstance(mgmt_ip_address, list):
                    for ip_address in mgmt_ip_address:
                        self._config_service(ip_address, key, conf_value)
                else:
                    self._config_service(mgmt_ip_address, key, conf_value)



# Copyright (C) 2021 Nippon Telegraph and Telephone Corporation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from datetime import datetime
import re

from dateutil import parser

from oslo_log import log as logging

from tacker.sol_refactored.common import exceptions as sol_ex
from tacker.sol_refactored.common import lcm_op_occ_utils as lcmocc_utils
from tacker.sol_refactored.common import subscription_utils as subsc_utils
from tacker.sol_refactored.common import vnf_instance_utils as inst_utils
from tacker.sol_refactored import objects


LOG = logging.getLogger(__name__)


class KeyAttribute(object):
    """A placeholder class for handling @key in filter attribute names"""

    def __str__(self):
        return "@key"


class FilterExpr(object):
    def __init__(self, op, attr, values):
        self.op = op
        self.attr = attr
        self.values = values

    def match_eq(self, val):
        return val == self.values[0]

    def match_neq(self, val):
        return val != self.values[0]

    def match_in(self, val):
        return val in self.values

    def match_nin(self, val):
        return val not in self.values

    def match_gt(self, val):
        return val > self.values[0]

    def match_gte(self, val):
        return val >= self.values[0]

    def match_lt(self, val):
        return val < self.values[0]

    def match_lte(self, val):
        return val <= self.values[0]

    def match_cont(self, val):
        for v in self.values:
            if v in val:
                return True
        return False

    def match_ncont(self, val):
        return not self.match_cont(val)

    def match(self, val):
        try:
            for a in self.attr:
                # NOTE(toshii): The attribute specified by "@key"
                # must be a dict, not a list of dicts. SOL013 isn't
                # very clear on this topic and the current implementation
                # doesn't support the latter.
                if isinstance(a, KeyAttribute):
                    val = list(val.keys())
                else:
                    val = val[a]
        except KeyError:
            LOG.debug("Attr %s not found in %s", self.attr, val)
            return False
        except (AttributeError, TypeError):
            raise sol_ex.InvalidAttributeFilter(
                sol_detail="AttrName %s is invalid" % '/'.join([
                    str(x) for x in self.attr]))
        LOG.debug("Key %s type %s", self.attr, type(val))
        # If not str, assume type conversion is already done.
        # Note: It is assumed that the type doesn't change between calls,
        # which can be problematic with KeyValuePairs.
        if isinstance(self.values[0], str):
            if isinstance(val, datetime):
                self.values[0] = parser.isoparse(self.values[0])
            elif isinstance(val, bool):
                if self.values[0] == 'true':
                    self.values[0] = True
                elif self.values[0] == 'false':
                    self.values[0] = False
                else:
                    raise sol_ex.InvalidAttributeFilter(
                        sol_detail="invalid value for boolean")
            elif isinstance(val, int):
                self.values = [int(v) for v in self.values]
            elif isinstance(val, float):
                self.values = [float(v) for v in self.values]

        # NOTE: This might not be SOL013 compliant when there are
        # multiple filters with a same attribute prefix.
        match_fn = getattr(self, "match_" + self.op)
        if isinstance(val, list):
            for v in val:
                if match_fn(v):
                    return True
            return False
        else:
            return match_fn(val)


class AttributeSelector(object):
    def __init__(self, default_exclude_list, all_fields=None, fields=None,
                 exclude_fields=None, exclude_default=None):
        self.exclude_fields = []
        self.fields = []
        if all_fields is not None:
            if fields is not None or exclude_fields is not None or \
               exclude_default is not None:
                raise sol_ex.InvalidAttributeSelector()
            # Nothing to do
        elif fields is not None:
            if exclude_fields is not None:
                raise sol_ex.InvalidAttributeSelector()
            self.fields = fields.split(',')
            if exclude_default is not None:
                self.exclude_fields = [v for v in default_exclude_list
                                       if v not in self.fields]
        elif exclude_fields is not None:
            if exclude_default is not None:
                raise sol_ex.InvalidAttributeSelector()
            self.exclude_fields = exclude_fields.split(',')
        else:
            self.exclude_fields = default_exclude_list

    def filter(self, obj, odict):
        deleted = {}
        if self.exclude_fields:
            excl_fields = self.exclude_fields
        else:
            if not self.fields:
                # Implies all_fields
                return odict
            excl_fields = [k for k in odict.keys() if k not in self.fields]

        for attr in excl_fields:
            klist = attr.split('/')

            # Check if the specified attribute has a lower cardinality bound
            # of 0. (SOL013 5.3.2.1)
            obj1 = obj
            for key in klist[:-1]:
                obj1 = getattr(obj1, key)
            if not obj1.fields[klist[-1]].nullable:
                continue

            val = odict
            deleted_ptr = deleted
            try:
                for i, k1 in enumerate(klist, start=1):
                    if i == len(klist):
                        deleted_ptr[k1] = val[k1]
                        del val[k1]
                    else:
                        val = val[k1]
                        if k1 not in deleted_ptr:
                            deleted_ptr[k1] = {}
                        deleted_ptr = deleted_ptr[k1]
            except (KeyError, TypeError):
                pass
        if not self.fields:
            return odict

        # Readd partial dictionary content
        for attr in self.fields:
            klist = attr.split('/')
            val = odict
            deleted_ptr = deleted
            try:
                for i, k1 in enumerate(klist, start=1):
                    if i == len(klist):
                        val[k1] = deleted_ptr[k1]
                    else:
                        if k1 not in val:
                            val[k1] = {}
                        val = val[k1]
                        deleted_ptr = deleted_ptr[k1]
            except KeyError:
                LOG.debug("Key %s not found in %s or %s", k1, val, deleted_ptr)
        return odict


class BaseViewBuilder(object):
    value_regexp = r"([^',)]+|('[^']*')+)"
    value_re = re.compile(value_regexp)
    simpleFilterExpr_re = re.compile(r"\(([a-z]+),([^,]+)(," +
                                     value_regexp + r")+\)")
    tildeEscape_re = re.compile(r"~([1ab])")
    opOne = ['eq', 'neq', 'gt', 'gte', 'lt', 'lte']
    opMulti = ['in', 'nin', 'cont', 'ncont']

    def __init__(self):
        pass

    def parse_attr(self, attr):
        def tilde_unescape(string):
            def repl(m):
                if m.group(1) == '1':
                    return '/'
                elif m.group(1) == 'a':
                    return ','
                elif m.group(1) == 'b':
                    return '@'

            if string == '@key':
                return KeyAttribute()
            s1 = self.tildeEscape_re.sub(repl, string)
            return re.sub('~0', '~', s1)

        attrs = attr.split('/')
        return [tilde_unescape(a) for a in attrs]

    def parse_values(self, values):
        loc = 0
        res = []
        while loc < len(values):
            if values[loc] != ",":
                LOG.debug("comma expected, %s at loc %d", values, loc)
                raise sol_ex.InvalidAttributeFilter(
                    sol_detail=("value parse error. comma expected, %s" %
                                values))
            loc += 1
            m = self.value_re.match(values[loc:])
            if m is None:
                LOG.debug("value parse error, %s at loc %d", values, loc)
                raise sol_ex.InvalidAttributeFilter(
                    sol_detail="value parse error")
            loc += m.end()
            if m.group(0).startswith("'"):
                res.append(re.sub("''", "'", m.group(0)[1:-1]))
            else:
                res.append(m.group(0))
        return res

    def parse_filter(self, filter):
        """Implement SOL013 5.2 Attribute-based filtering"""

        loc = 0
        res = []
        while True:
            m = self.simpleFilterExpr_re.match(filter[loc:])
            if m is None:
                LOG.debug("filter %s parse error at char %d", filter, loc)
                raise sol_ex.InvalidAttributeFilter(
                    sol_detail="filter parse error")
            op = m.group(1)
            if op not in self.opOne and op not in self.opMulti:
                raise sol_ex.InvalidAttributeFilter(
                    sol_detail=("Invalid op %s" % op))
            values = self.parse_values(
                filter[(loc + m.end(2)):(loc + m.end(3))])
            if len(values) > 1 and op not in self.opMulti:
                raise sol_ex.InvalidAttributeFilter(
                    sol_detail=("Only one value is allowed for op %s" % op))
            res.append(FilterExpr(op, self.parse_attr(m.group(2)), values))
            loc += m.end()
            if loc == len(filter):
                return res
            if filter[loc] != ';':
                LOG.debug("filter %s parse error at char %d "
                          "(semicolon expected)", filter, loc)
                raise sol_ex.InvalidAttributeFilter(
                    sol_detail="filter parse error. semicolon expected.")
            loc += 1

    def parse_selector(self, req):
        """Implement SOL013 5.3 Attribute selectors"""
        params = {}
        for k in ['all_fields', 'fields', 'exclude_fields', 'exclude_default']:
            v = req.get(k)
            if v is not None:
                params[k] = v
        return AttributeSelector(self._EXCLUDE_DEFAULT, **params)

    def match_filters(self, val, filters):
        if filters is None:
            return True

        for f in filters:
            if not f.match(val):
                return False
        return True

    def detail_list(self, values, filters, selector):
        return [self.detail(v, selector) for v in values
                if self.match_filters(v, filters)]


class InstanceViewBuilder(BaseViewBuilder):
    _EXCLUDE_DEFAULT = ['vnfConfigurableProperties',
                        'vimConnectionInfo',
                        'instantiatedVnfInfo',
                        'metadata',
                        'extensions']

    def __init__(self, endpoint):
        self.endpoint = endpoint

    def parse_filter(self, filter):
        return super().parse_filter(filter)

    def detail(self, inst, selector=None):
        # NOTE: _links is not saved in DB. create when it is necessary.
        if not inst.obj_attr_is_set('_links'):
            inst._links = inst_utils.make_inst_links(inst, self.endpoint)

        resp = inst.to_dict()

        # remove password from vim_connection_info
        # see SOL003 4.4.1.6
        for vim_info in resp.get('vimConnectionInfo', {}).values():
            if ('accessInfo' in vim_info and
                    'password' in vim_info['accessInfo']):
                vim_info['accessInfo'].pop('password')

        if selector is not None:
            resp = selector.filter(inst, resp)
        return resp

    def detail_list(self, insts, filters, selector):
        return super().detail_list(insts, filters, selector)


class LcmOpOccViewBuilder(BaseViewBuilder):
    _EXCLUDE_DEFAULT = ['operationParams',
                        'error',
                        'resourceChanges',
                        'changedInfo',
                        'changedExtConnectivity']

    def __init__(self, endpoint):
        self.endpoint = endpoint

    def parse_filter(self, filter):
        return super().parse_filter(filter)

    def detail(self, lcmocc, selector=None):
        # NOTE: _links is not saved in DB. create when it is necessary.
        if not lcmocc.obj_attr_is_set('_links'):
            lcmocc._links = lcmocc_utils.make_lcmocc_links(lcmocc,
                                                           self.endpoint)
        resp = lcmocc.to_dict()
        if selector is not None:
            resp = selector.filter(lcmocc, resp)
        return resp

    def detail_list(self, lcmoccs, filters, selector):
        return super().detail_list(lcmoccs, filters, selector)


class SubscriptionViewBuilder(BaseViewBuilder):
    def __init__(self, endpoint):
        self.endpoint = endpoint

    def parse_filter(self, filter):
        return super().parse_filter(filter)

    def detail(self, subsc, selector=None):
        # NOTE: _links is not saved in DB. create when it is necessary.
        if not subsc.obj_attr_is_set('_links'):
            self_href = subsc_utils.subsc_href(subsc.id, self.endpoint)
            subsc._links = objects.LccnSubscriptionV2_Links()
            subsc._links.self = objects.Link(href=self_href)

        resp = subsc.to_dict()

        # NOTE: authentication is not included in LccnSubscription
        resp.pop('authentication', None)

        if selector is not None:
            resp = selector.filter(subsc, resp)
        return resp

    def detail_list(self, subscs, filters):
        return super().detail_list(subscs, filters, None)



#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

from datetime import datetime
from unittest import mock

from oslo_utils import uuidutils

from tacker import context
from tacker.db.nfvo import nfvo_db
from tacker.objects import heal_vnf_request
from tacker.tests.unit.db import base as db_base
from tacker.vnfm import plugin
from tacker.vnfm.policy_actions.vdu_autoheal import vdu_autoheal


vnf_dict = {
    'id': uuidutils.generate_uuid(),
    'mgmt_ip_address': '{"VDU1": "a.b.c.d"}',
    'vim_id': '6261579e-d6f3-49ad-8bc3-a9cb974778ff',
    'instance_id': 'a737497c-761c-11e5-89c3-9cb6541d805d',
    'attributes': {
        'heat_template': {
            'resources': {
                'VDU1': {
                    'properties': {
                        'networks': [{'port': {'get_resource': 'CP1'}}]}
                }
            }
        }
    }
}


class FakeDriverManager(mock.Mock):
    def invoke(self, *args, **kwargs):
        if 'create' in args:
            return uuidutils.generate_uuid()

        if 'get_resource_info' in args:
            return {'resources': {'name': 'dummy_vnf',
                                  'type': 'dummy',
                                  'id': uuidutils.generate_uuid()}}


class FakeVNFMonitor(mock.Mock):
    pass


class TestVNFActionVduAutoheal(db_base.SqlTestCase):

    def setUp(self):
        super(TestVNFActionVduAutoheal, self).setUp()
        self.context = context.get_admin_context()
        self._mock_device_manager()
        self._mock_vnf_monitor()
        self._insert_dummy_vim()
        self.vnfm_plugin = plugin.VNFMPlugin()
        self.vdu_autoheal = vdu_autoheal.VNFActionVduAutoheal()
        self.addCleanup(mock.patch.stopall)

    def _mock_device_manager(self):
        self._device_manager = mock.Mock(wraps=FakeDriverManager())
        self._device_manager.__contains__ = mock.Mock(
            return_value=True)
        fake_device_manager = mock.Mock()
        fake_device_manager.return_value = self._device_manager
        self._mock(
            'tacker.common.driver_manager.DriverManager', fake_device_manager)

    def _mock_vnf_monitor(self):
        self._vnf_monitor = mock.Mock(wraps=FakeVNFMonitor())
        fake_vnf_monitor = mock.Mock()
        fake_vnf_monitor.return_value = self._vnf_monitor
        self._mock(
            'tacker.vnfm.monitor.VNFMonitor', fake_vnf_monitor)

    def _insert_dummy_vim(self):
        session = self.context.session
        vim_db = nfvo_db.Vim(
            id='6261579e-d6f3-49ad-8bc3-a9cb974778ff',
            tenant_id='ad7ebc56538745a08ef7c5e97f8bd437',
            name='fake_vim',
            description='fake_vim_description',
            type='test_vim',
            status='Active',
            deleted_at=datetime.min,
            placement_attr={'regions': ['RegionOne']})
        vim_auth_db = nfvo_db.VimAuth(
            vim_id='6261579e-d6f3-49ad-8bc3-a9cb974778ff',
            password='encrypted_pw',
            auth_url='http://localhost:5000',
            vim_project={'name': 'test_project'},
            auth_cred={'username': 'test_user', 'user_domain_id': 'default',
                       'project_domain_id': 'default'})
        session.add(vim_db)
        session.add(vim_auth_db)
        session.flush()

    @mock.patch('tacker.vnfm.plugin.VNFMPlugin.heal_vnf')
    @mock.patch('yaml.safe_load')
    @mock.patch('tacker.objects.HealVnfRequest')
    def test_vdu_autoheal_execute_action(self, mock_heal_vnf_request,
                                         mock_safe_load,
                                         mock_heal_vnf):
        # Here yaml.safe_load is mock as in the test case i am passing
        # vnf_dict containing having vnf_dict['attributes']['heat_template']
        # value in json format so while excution it giving the error as
        # dict object has no read attribute where as in actual execution the
        # value of vnf_dict['attributes']['heat_template'] is in ymal format.
        mock_safe_load.return_value = vnf_dict['attributes']['heat_template']
        resource_list = ['VDU1', 'CP1']
        additional_params = []
        for resource in resource_list:
            additional_paramas_obj = heal_vnf_request.HealVnfAdditionalParams(
                parameter=resource,
                cause=["Unable to reach while monitoring resource: '%s'" %
                       resource])
            additional_params.append(additional_paramas_obj)
        heal_request_data_obj = heal_vnf_request.HealVnfRequest(
            cause='VNF monitoring fails.',
            legacy_additional_params=additional_params)
        mock_heal_vnf_request.return_value = heal_request_data_obj
        self.vdu_autoheal.execute_action(self.vnfm_plugin, self.context,
                                         vnf_dict, args={'vdu_name': 'VDU1'})
        mock_heal_vnf.assert_called_once_with(self.context, vnf_dict['id'],
                                              heal_request_data_obj)

    @mock.patch('tacker.vnfm.policy_actions.vdu_autoheal.'
                'vdu_autoheal.LOG')
    def test_vdu_autoheal_action_with_no_vdu_name(self, mock_log):
        expected_error_msg = ("VDU resource of vnf '%s' is not present for "
                              "autoheal." % vnf_dict['id'])
        self.vdu_autoheal.execute_action(self.vnfm_plugin, self.context,
                                         vnf_dict, args={})
        mock_log.error.assert_called_with(expected_error_msg)



# Copyright (C) 2021 Nippon Telegraph and Telephone Corporation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tacker.sol_refactored.objects import base
from tacker.sol_refactored.objects import fields
from tacker.sol_refactored.objects.v2 import fields as v2fields


# NFV-SOL 003
# - v3.3.1 5.5.2.17 (API version: 2.0.0)
@base.TackerObjectRegistry.register
class VnfLcmOperationOccurrenceNotificationV2(base.TackerObject,
                                              base.TackerObjectDictCompat):

    # Version 1.0: Initial version
    VERSION = '1.0'

    fields = {
        'id': fields.StringField(nullable=False),
        'notificationType': fields.StringField(nullable=False),
        'subscriptionId': fields.StringField(nullable=False),
        'timeStamp': fields.DateTimeField(nullable=False),
        'notificationStatus': fields.EnumField(
            valid_values=['START', 'RESULT'], nullable=False),
        'operationState': v2fields.LcmOperationStateTypeField(
            nullable=False),
        'vnfInstanceId': fields.StringField(nullable=False),
        'operation': v2fields.LcmOperationTypeField(nullable=False),
        'isAutomaticInvocation': fields.BooleanField(nullable=False),
        'verbosity': v2fields.LcmOpOccNotificationVerbosityTypeField(
            nullable=True),
        'vnfLcmOpOccId': fields.StringField(nullable=False),
        'affectedVnfcs': fields.ListOfObjectsField(
            'AffectedVnfcV2', nullable=True),
        'affectedVirtualLinks': fields.ListOfObjectsField(
            'AffectedVirtualLinkV2', nullable=True),
        'affectedExtLinkPorts': fields.ListOfObjectsField(
            'AffectedExtLinkPortV2', nullable=True),
        'affectedVirtualStorages': fields.ListOfObjectsField(
            'AffectedVirtualStorageV2', nullable=True),
        'changedInfo': fields.ObjectField(
            'VnfInfoModificationsV2', nullable=True),
        'changedExtConnectivity': fields.ListOfObjectsField(
            'ExtVirtualLinkInfoV2', nullable=True),
        'modificationsTriggeredByVnfPkgChange': fields.ObjectField(
            'ModificationsTriggeredByVnfPkgChangeV2', nullable=True),
        'error': fields.ObjectField('ProblemDetails', nullable=True),
        '_links': fields.ObjectField('LccnLinksV2', nullable=False),
    }



# Copyright 2016 Brocade Communications System, Inc.
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from unittest import mock

from oslo_config import cfg
from oslo_config import fixture as config_fixture
from requests_mock.contrib import fixture as requests_mock_fixture

from tacker.tests import base
from tacker.tests.unit import fixtures as tacker_fixtures

CONF = cfg.CONF


class TestCase(base.BaseTestCase):

    def setUp(self):
        super(TestCase, self).setUp()
        self.config_fixture = self.useFixture(config_fixture.Config(CONF))
        CONF([], default_config_files=[])

        # Limit the amount of DeprecationWarning messages in the unit test logs
        self.useFixture(tacker_fixtures.WarningsFixture())

    def _mock(self, target, new=mock.DEFAULT):
        patcher = mock.patch(target, new)
        return patcher.start()

    def compare_obj(self, expected, result, subs=None, allow_missing=None):
        if subs is None:
            subs = {}
        if allow_missing is None:
            allow_missing = []

        for key in expected.fields:
            if key in allow_missing:
                continue
            obj_val = getattr(expected, key)
            db_key = subs.get(key, key)
            db_val = getattr(result, db_key)

            self.assertEqual(db_val, obj_val)


class FixturedTestCase(TestCase):
    client_fixture_class = None
    sdk_connection_fixure_class = None

    def setUp(self):
        super(FixturedTestCase, self).setUp()
        if self.client_fixture_class or self.sdk_connection_fixure_class:
            self.requests_mock = self.useFixture(requests_mock_fixture.
                                                 Fixture())
        if self.client_fixture_class:
            hc_fix = self.client_fixture_class(self.requests_mock)
            self.cs = self.useFixture(hc_fix).client

        if self.sdk_connection_fixure_class:
            sdk_conn_fix = self.sdk_connection_fixure_class(self.requests_mock)
            self.sdk_conn = self.useFixture(sdk_conn_fix).client



# Copyright (C) 2019 NTT DATA
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from copy import deepcopy
import ddt
import os
import tempfile
import time
import urllib
import zipfile

from oslo_serialization import jsonutils

import tacker.conf
from tacker.tests.functional import base
from tacker.tests import utils


CONF = tacker.conf.CONF


@ddt.ddt
class VnfPackageTest(base.BaseTackerTest):

    VNF_PACKAGE_DELETE_TIMEOUT = 120
    VNF_PACKAGE_UPLOAD_TIMEOUT = 300

    # The size of CSAR zip file used while verifying downloaded package.
    # NOTE: It should be updated if some contents of packages under
    #       `tacker/tests/etc/samples` dir.
    SIZE_CSAR_ZIP = 16389096

    def setUp(self):
        super(VnfPackageTest, self).setUp()
        self.base_url = "/vnfpkgm/v1/vnf_packages"

        # Here we create and upload vnf package. Also get 'show' api response
        # as reference data for attribute filter tests
        self.package_id1 = self._create_and_upload_vnf("vnfpkgm1")
        show_url = self.base_url + "/" + self.package_id1
        resp, self.package1 = self.http_client.do_request(show_url, "GET")
        self.assertEqual(200, resp.status_code)

        self.package_id2 = self._create_and_upload_vnf("vnfpkgm2")
        show_url = self.base_url + "/" + self.package_id2
        resp, self.package2 = self.http_client.do_request(show_url, "GET")
        self.assertEqual(200, resp.status_code)

        self.package_id3 = self._create_and_upload_vnf("vnfpkgm3")
        show_url = self.base_url + "/" + self.package_id3
        resp, self.package3 = self.http_client.do_request(show_url, "GET")
        self.assertEqual(200, resp.status_code)

    def tearDown(self):
        for package_id in [self.package_id1, self.package_id2,
                           self.package_id3]:
            self._disable_operational_state(package_id)
            self._delete_vnf_package(package_id)
            self._wait_for_delete(package_id)

        super(VnfPackageTest, self).tearDown()

    def _wait_for_delete(self, package_uuid):
        show_url = self.base_url + "/" + package_uuid
        timeout = self.VNF_PACKAGE_DELETE_TIMEOUT
        start_time = int(time.time())
        while True:
            resp, body = self.http_client.do_request(show_url, "GET")
            if (404 == resp.status_code):
                break
            if (int(time.time()) - start_time) > timeout:
                raise Exception("Failed to delete package")
            time.sleep(1)

    def _wait_for_onboard(self, package_uuid):
        show_url = self.base_url + "/" + package_uuid
        timeout = self.VNF_PACKAGE_UPLOAD_TIMEOUT
        start_time = int(time.time())
        while True:
            resp, body = self.http_client.do_request(show_url, "GET")
            if body['onboardingState'] == "ONBOARDED":
                break

            if ((int(time.time()) - start_time) > timeout):
                raise Exception("Failed to onboard vnf package")

            time.sleep(1)

    def _create_vnf_package(self, body):
        resp, response_body = self.http_client.do_request(self.base_url,
                                                   "POST", body=body)
        self.assertIsNotNone(response_body['id'])
        self.assertEqual(201, resp.status_code)
        return response_body

    def _disable_operational_state(self, package_uuid):
        update_req_body = jsonutils.dumps({
            "operationalState": "DISABLED"})

        resp, resp_body = self.http_client.do_request(
            '{base_path}/{id}'.format(id=package_uuid,
                                      base_path=self.base_url),
            "PATCH", content_type='application/json', body=update_req_body)
        self.assertEqual(200, resp.status_code)

    def _delete_vnf_package(self, package_uuid):
        url = self.base_url + "/" + package_uuid
        resp, body = self.http_client.do_request(url, "DELETE")
        self.assertEqual(204, resp.status_code)

    def test_create_show_delete_vnf_package(self):
        """Creates and deletes a vnf package."""

        # Create vnf package
        vnf_package_id = self._create_and_upload_vnf('vnfpkgm1')

        # show vnf package
        show_url = self.base_url + "/" + vnf_package_id
        resp, body = self.http_client.do_request(show_url, "GET")
        self.assertEqual(200, resp.status_code)

        # update vnf package
        self._disable_operational_state(vnf_package_id)

        # Delete vnf package
        self._delete_vnf_package(vnf_package_id)
        self._wait_for_delete(vnf_package_id)

        # show vnf package should fail as it's deleted
        resp, body = self.http_client.do_request(show_url, "GET")
        self.assertEqual(404, resp.status_code)

    def test_list(self):
        vnf_package_list = []
        body = jsonutils.dumps({"userDefinedData": {"foo": "bar"}})

        # create two vnf packages
        vnf_package = self._create_vnf_package(body)
        self.addCleanup(self._delete_vnf_package, vnf_package['id'])
        vnf_package_list.append(vnf_package['id'])

        vnf_package = self._create_vnf_package(body)
        vnf_package_list.append(vnf_package['id'])
        self.addCleanup(self._delete_vnf_package, vnf_package['id'])

        # list vnf package
        resp, body = self.http_client.do_request(self.base_url, "GET")
        self.assertEqual(200, resp.status_code)

        package_uuids = [obj['id'] for obj in body]
        self.assertIn(vnf_package_list[0], package_uuids)
        self.assertIn(vnf_package_list[1], package_uuids)

    def _get_csar_dir_path(self, csar_name):
        csar_dir = os.path.abspath(os.path.join(os.path.dirname(__file__),
                                   "../../../etc/samples/etsi/nfv", csar_name))
        return csar_dir

    def _create_and_upload_vnf(self, sample_name):
        body = jsonutils.dumps({"userDefinedData": {"foo": "bar"}})
        vnf_package = self._create_vnf_package(body)
        csar_dir = self._get_csar_dir_path(sample_name)
        if os.path.exists(os.path.join(csar_dir, 'TOSCA-Metadata')) and \
                sample_name != 'vnfpkgm2':
            file_path = utils.create_csar_with_unique_artifact(
                csar_dir)
        else:
            file_path, vnfd_id = utils.create_csar_with_unique_vnfd_id(
                csar_dir)
        self.addCleanup(os.remove, file_path)

        with open(file_path, 'rb') as file_object:
            resp, resp_body = self.http_client.do_request(
                '{base_path}/{id}/package_content'.format(
                    id=vnf_package['id'],
                    base_path=self.base_url),
                "PUT", body=file_object, content_type='application/zip')

        self.assertEqual(202, resp.status_code)

        self._wait_for_onboard(vnf_package['id'])

        return vnf_package['id']

    def test_upload_from_uri_without_auth_and_delete(self):
        csar_dir = self._get_csar_dir_path("sample_vnfpkg_no_meta_single_vnfd")
        file_path, vnfd_id = utils.create_csar_with_unique_vnfd_id(csar_dir)
        self.addCleanup(os.remove, file_path)

        cls_obj = utils.StaticHttpFileHandler(os.path.dirname(file_path))
        self.addCleanup(cls_obj.stop)

        body = jsonutils.dumps({"userDefinedData": {"foo": "bar"}})
        vnf_package = self._create_vnf_package(body)
        csar_file_uri = 'http://localhost:{port}/{filename}'.format(
            port=cls_obj.port, filename=os.path.basename(file_path))

        body = jsonutils.dumps({"addressInformation": csar_file_uri})
        resp, resp_body = self.http_client.do_request(
            '{base_path}/{id}/package_content/upload_from_uri'.format(
                id=vnf_package['id'],
                base_path=self.base_url),
            "POST", body=body)
        self.assertEqual(202, resp.status_code)

        self._wait_for_onboard(vnf_package['id'])

        self._disable_operational_state(vnf_package['id'])
        self._delete_vnf_package(vnf_package['id'])
        self._wait_for_delete(vnf_package['id'])

    def test_upload_from_uri_with_auth_and_delete(self):
        csar_dir = self._get_csar_dir_path("sample_vnfpkg_no_meta_single_vnfd")
        file_path, vnfd_id = utils.create_csar_with_unique_vnfd_id(csar_dir)
        self.addCleanup(os.remove, file_path)

        cls_obj = utils.StaticHttpFileHandler(os.path.dirname(file_path))
        self.addCleanup(cls_obj.stop)

        body = jsonutils.dumps({"userDefinedData": {"foo": "bar"}})
        vnf_package = self._create_vnf_package(body)
        csar_file_uri = 'http://localhost:{port}/{filename}'.format(
            port=cls_obj.port, filename=os.path.basename(file_path))
        body = jsonutils.dumps({"addressInformation": csar_file_uri,
                                "userName": "username",
                                "password": "password"})
        resp, resp_body = self.http_client.do_request(
            '{base_path}/{id}/package_content/upload_from_uri'.format(
                id=vnf_package['id'],
                base_path=self.base_url),
            "POST", body=body)
        self.assertEqual(202, resp.status_code)

        self._wait_for_onboard(vnf_package['id'])

        self._disable_operational_state(vnf_package['id'])
        self._delete_vnf_package(vnf_package['id'])
        self._wait_for_delete(vnf_package['id'])

    def test_patch_in_onboarded_state(self):
        user_data = jsonutils.dumps(
            {"userDefinedData": {"key1": "val1", "key2": "val2",
                                 "key3": "val3"}})
        vnf_package = self._create_vnf_package(user_data)

        update_req_body = jsonutils.dumps(
            {"operationalState": "DISABLED",
             "userDefinedData": {"key1": "changed_val1",
                                 "key2": "val2", "new_key": "new_val"}})

        expected_result = {"operationalState": "DISABLED",
                           "userDefinedData": {
                               "key1": "changed_val1", "new_key": "new_val"}}

        csar_dir = self._get_csar_dir_path("vnfpkgm1")
        file_path = utils.create_csar_with_unique_artifact(csar_dir)
        self.addCleanup(os.remove, file_path)
        with open(file_path, 'rb') as file_object:
            resp, resp_body = self.http_client.do_request(
                '{base_path}/{id}/package_content'.format(
                    id=vnf_package['id'],
                    base_path=self.base_url),
                "PUT", body=file_object, content_type='application/zip')

        self.assertEqual(202, resp.status_code)
        self._wait_for_onboard(vnf_package['id'])

        # Update vnf package which is onboarded
        resp, resp_body = self.http_client.do_request(
            '{base_path}/{id}'.format(id=vnf_package['id'],
                                      base_path=self.base_url),
            "PATCH", content_type='application/json', body=update_req_body)

        self.assertEqual(200, resp.status_code)
        self.assertEqual(expected_result, resp_body)
        self._delete_vnf_package(vnf_package['id'])
        self._wait_for_delete(vnf_package['id'])

    def test_index_attribute_filter(self):
        filter_expr = {
            'filter': "(gt,softwareImages/minDisk,7);"
            "(eq,onboardingState,ONBOARDED);"
            "(eq,softwareImages/checksum/algorithm,'sha-512');"
            "(eq,additionalArtifacts/checksum/algorithm,'sha-256')"
        }
        filter_url = self.base_url + "?" + urllib.parse.urlencode(filter_expr)
        resp, body = self.http_client.do_request(filter_url, "GET")
        package = deepcopy(self.package2)
        for attr in ['softwareImages', 'checksum', 'userDefinedData',
                     'additionalArtifacts']:
            package.pop(attr, None)
        expected_result = [package]
        self.assertEqual(expected_result, body)

    def test_index_attribute_selector_all_fields(self):
        """Test for attribute selector 'all_fields'

        We intentionally use attribute filter along with attribute selector.
        It is because when these tests run with concurrency > 1, there will
        be multiple sample packages present at a time. Hence attribute
        selector will be applied on all of them. It will be difficult to
        predict the expected result. Hence we are limiting the result set by
        filtering one of the vnf package which was created for this speific
        test.
        """
        filter_expr = {'filter': '(eq,id,%s)' % self.package_id1,
            'all_fields': ''}
        filter_url = self.base_url + "?" + urllib.parse.urlencode(filter_expr)
        resp, body = self.http_client.do_request(filter_url, "GET")
        expected_result = [self.package1]
        self.assertEqual(expected_result, body)

    def test_index_attribute_selector_exclude_default(self):
        filter_expr = {'filter': '(eq,id,%s)' % self.package_id2,
            'exclude_default': ''}
        filter_url = self.base_url + "?" + urllib.parse.urlencode(filter_expr)
        resp, body = self.http_client.do_request(filter_url, "GET")
        package2 = deepcopy(self.package2)
        for attr in ['softwareImages', 'checksum', 'userDefinedData',
                     'additionalArtifacts']:
            package2.pop(attr, None)
        expected_result = [package2]
        self.assertEqual(expected_result, body)

    def test_index_attribute_selector_exclude_fields(self):
        filter_expr = {
            'filter': '(eq,id,%s)' % self.package_id2,
            'exclude_fields': 'checksum,softwareImages/checksum,'
                              'additionalArtifacts/checksum'}
        filter_url = self.base_url + "?" + urllib.parse.urlencode(filter_expr)
        resp, body = self.http_client.do_request(filter_url, "GET")
        package2 = deepcopy(self.package2)
        for software_image in package2['softwareImages']:
            software_image.pop('checksum', None)
        for artifact in package2['additionalArtifacts']:
            artifact.pop('checksum', None)
        package2.pop('checksum', None)
        expected_result = [package2]
        self.assertEqual(expected_result, body)

    def test_index_attribute_selector_fields(self):
        filter_expr = {'filter': '(eq,id,%s)' % self.package_id1,
            'fields': 'softwareImages/checksum/hash,'
            'softwareImages/containerFormat,softwareImages/name,'
            'userDefinedData,additionalArtifacts/checksum/hash,'
            'additionalArtifacts/artifactPath'}
        filter_url = self.base_url + "?" + urllib.parse.urlencode(filter_expr)
        resp, body = self.http_client.do_request(filter_url, "GET")
        package1 = deepcopy(self.package1)

        # Prepare expected result
        for software_image in package1['softwareImages']:
            software_image['checksum'].pop('algorithm', None)
            for attr in ['createdAt', 'diskFormat', 'id', 'imagePath',
                    'minDisk', 'minRam', 'provider', 'size', 'userMetadata',
                    'version']:
                software_image.pop(attr, None)
        for artifact in package1['additionalArtifacts']:
            artifact['checksum'].pop('algorithm', None)
            artifact.pop('metadata', None)
        package1.pop('checksum', None)
        expected_result = [package1]
        self.assertEqual(expected_result, body)

    def test_get_vnfd_from_onboarded_vnf_package_for_content_type_zip(self):
        resp, resp_body = self.http_client.do_request(
            '{base_path}/{id}/vnfd'.format(id=self.package_id1,
                                           base_path=self.base_url),
            "GET", content_type='application/zip')
        self.assertEqual(200, resp.status_code)
        self.assertEqual('application/zip', resp.headers['Content-Type'])
        self.assert_resp_contents(resp)

    def assert_resp_contents(self, resp):
        expected_file_list = ['Definitions/helloworld3_top.vnfd.yaml',
                              'Definitions/helloworld3_df_simple.yaml',
                              'Definitions/etsi_nfv_sol001_vnfd_types.yaml',
                              'Definitions/etsi_nfv_sol001_common_types.yaml',
                              'Definitions/helloworld3_types.yaml',
                              'TOSCA-Metadata/TOSCA.meta']

        tmp = tempfile.NamedTemporaryFile(delete=False)
        try:
            tmp.write(resp.content)
        finally:
            # checking response.content is valid zip file
            self.assertTrue(zipfile.is_zipfile(tmp))
            with zipfile.ZipFile(tmp, 'r') as zipObj:
                # Get list of files names in zip
                actual_file_list = zipObj.namelist()
            self.assertCountEqual(expected_file_list, actual_file_list)

            tmp.close()

    @ddt.data('text/plain', 'application/zip,text/plain')
    def test_get_vnfd_from_onboarded_vnf_package_for_content_type_text(
            self, accept_header):
        # Uploading vnf package with single yaml file csar.
        single_yaml_csar_dir = "sample_vnfpkg_no_meta_single_vnfd"
        vnf_package_id = self._create_and_upload_vnf(
            single_yaml_csar_dir)
        self.addCleanup(self._delete_vnf_package, vnf_package_id)
        self.addCleanup(self._disable_operational_state, vnf_package_id)
        resp, resp_body = self.http_client.do_request(
            '{base_path}/{id}/vnfd'.format(id=vnf_package_id,
                                           base_path=self.base_url),
            "GET", content_type=accept_header)
        self.assertEqual(200, resp.status_code)
        self.assertEqual('text/plain', resp.headers['Content-Type'])
        self.assertIsNotNone(resp.text)

    def test_fetch_vnf_package_content_partial_download_using_range(self):
        """Test partial download using 'Range' requests for csar zip"""
        # test for success on satisfiable Range request.
        range_ = 'bytes=3-8'
        headers = {'Range': range_}
        response = self.http_client.do_request(
            '{base_path}/{id}/package_content'.format(
                id=self.package_id1, base_path=self.base_url),
            "GET", body={}, headers=headers)
        self.assertEqual(206, response[0].status_code)
        self.assertEqual(
            '\x04\x14\x00\x00\x00\x00', response[0].content.decode(
                'utf-8', 'ignore'))
        self.assertEqual('6', response[0].headers['Content-Length'])

    def test_fetch_vnf_package_content_full_download(self):
        """Test full download for csar zip"""
        response = self.http_client.do_request(
            '{base_path}/{id}/package_content'.format(
                id=self.package_id1, base_path=self.base_url),
            "GET", body={}, headers={})
        self.assertEqual(200, response[0].status_code)
        self.assertEqual(
            str(self.SIZE_CSAR_ZIP), response[0].headers['Content-Length'])

    def test_fetch_vnf_package_content_combined_download(self):
        """Combine two partial downloads using 'Range' requests for csar zip"""

        zip_file_path = tempfile.NamedTemporaryFile(delete=True)
        zipf = zipfile.ZipFile(zip_file_path, 'w', zipfile.ZIP_STORED)

        # Partial download 1
        range_ = 'bytes=0-10'
        headers = {'Range': range_}
        response_1 = self.http_client.do_request(
            '{base_path}/{id}/package_content'.format(
                id=self.package_id1, base_path=self.base_url),
            "GET", body={}, headers=headers)
        size_1 = int(response_1[0].headers['Content-Length'])
        data = response_1[0].content
        file_path = self._get_csar_dir_path("data.txt")
        zipf.writestr(file_path, data)

        # Partial download 2
        range_ = 'bytes=11-{}'.format(self.SIZE_CSAR_ZIP)
        headers = {'Range': range_}
        response_2 = self.http_client.do_request(
            '{base_path}/{id}/package_content'.format(
                id=self.package_id1, base_path=self.base_url),
            "GET", body={}, headers=headers)

        data = response_2[0].content
        zipf.writestr(file_path, data)
        zipf.close()
        size_2 = int(response_2[0].headers['Content-Length'])
        total_size = size_1 + size_2
        self.assertEqual(True, zipfile.is_zipfile(zip_file_path))
        self.assertEqual(self.SIZE_CSAR_ZIP, total_size)
        zip_file_path.close()

    def test_fetch_vnf_package_artifacts(self):
        # run download api
        response1 = self.http_client.do_request(
            '{base_path}/{id}/artifacts/{artifact_path}'.format(
                base_path=self.base_url, id=self.package_id1,
                artifact_path='Scripts/install.sh'),
            "GET", body={}, headers={})

        response2 = self.http_client.do_request(
            '{base_path}/{id}/artifacts/{artifact_path}'.format(
                base_path=self.base_url, id=self.package_id2,
                artifact_path='Scripts/install.sh'),
            "GET", body={}, headers={})

        response3 = self.http_client.do_request(
            '{base_path}/{id}/artifacts/{artifact_path}'.format(
                base_path=self.base_url, id=self.package_id3,
                artifact_path='Scripts/install.sh'),
            "GET", body={}, headers={})
        # verification
        self.assertEqual(200, response1[0].status_code)
        self.assertEqual('33', response1[0].headers['Content-Length'])
        self.assertIsNotNone(response1[1])
        self.assertEqual(200, response2[0].status_code)
        self.assertEqual('33', response2[0].headers['Content-Length'])
        self.assertIsNotNone(response2[1])
        self.assertEqual(200, response3[0].status_code)
        self.assertEqual('33', response3[0].headers['Content-Length'])
        self.assertIsNotNone(response3[1])

    def test_fetch_vnf_package_artifacts_partial_download_using_range(self):
        # get range
        range_ = 'bytes=3-8'
        # get headers
        headers = {'Range': range_}
        # request download api
        response = self.http_client.do_request(
            '{base_path}/{id}/artifacts/{artifact_path}'.format(
                base_path=self.base_url, id=self.package_id1,
                artifact_path='Scripts/install.sh'),
            "GET", body={}, headers=headers)
        # verification
        self.assertEqual(206, response[0].status_code)
        self.assertEqual('6', response[0].headers['Content-Length'])
        self.assertIsNotNone(response[1])

    def test_vnf_package_software_image_path(self):
        filter_expr = {
            'filter': '(eq,id,%s)' % self.package_id2,
            'exclude_fields': 'checksum,softwareImages/checksum,'
                              'additionalArtifacts/checksum'}
        filter_url = self.base_url + "?" + urllib.parse.urlencode(filter_expr)
        resp, body = self.http_client.do_request(filter_url, "GET")
        image_path = body[0]['softwareImages'][0]['imagePath']
        expected_result = self.package2.get('softwareImages')[0]['imagePath']
        self.assertEqual(image_path, expected_result)



# Copyright (C) 2021 Nippon Telegraph and Telephone Corporation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tacker._i18n import _
from tacker.sol_refactored import objects
from tacker.sol_refactored.objects import base
from tacker.sol_refactored.objects import fields
from tacker.sol_refactored.objects.v2 import fields as v2fields


class OperationParam(fields.FieldType):
    @staticmethod
    def coerce(obj, attr, value):
        if not hasattr(obj, 'operation'):
            raise ValueError(_("'operation' must have been coerced "
                               "before 'operationParams'"))
        if obj.operation == v2fields.LcmOperationType.INSTANTIATE:
            cls = objects.InstantiateVnfRequest
        elif obj.operation == v2fields.LcmOperationType.SCALE:
            cls = objects.ScaleVnfRequest
        elif obj.operation == v2fields.LcmOperationType.SCALE_TO_LEVEL:
            cls = objects.ScaleVnfToLevelRequest
        elif obj.operation == v2fields.LcmOperationType.CHANGE_FLAVOUR:
            cls = objects.ChangeVnfFlavourRequest
        elif obj.operation == v2fields.LcmOperationType.OPERATE:
            cls = objects.OperateVnfRequest
        elif obj.operation == v2fields.LcmOperationType.HEAL:
            cls = objects.HealVnfRequest
        elif obj.operation == v2fields.LcmOperationType.CHANGE_EXT_CONN:
            cls = objects.ChangeExtVnfConnectivityRequest
        elif obj.operation == v2fields.LcmOperationType.TERMINATE:
            cls = objects.TerminateVnfRequest
        elif obj.operation == v2fields.LcmOperationType.MODIFY_INFO:
            cls = objects.VnfInfoModificationRequest
        elif obj.operation == v2fields.LcmOperationType.CREATE_SNAPSHOT:
            cls = objects.CreateVnfSnapshotRequest
        elif obj.operation == v2fields.LcmOperationType.REVERT_TO_SNAPSHOT:
            cls = objects.RevertToVnfSnapshotRequest
        elif obj.operation == v2fields.LcmOperationType.CHANGE_VNFPKG:
            cls = objects.ChangeCurrentVnfPkgRequest
        else:
            raise ValueError(_("Unexpected 'operation' found."))

        return cls.from_dict(value)

    def to_primitive(self, obj, attr, value):
        return value.to_dict()


class OperationParamsField(fields.AutoTypedField):
    AUTO_TYPE = OperationParam()


# NFV-SOL 003
# - v3.3.1 5.5.2.13 (API version: 2.0.0)
@base.TackerObjectRegistry.register
class VnfLcmOpOccV2(base.TackerPersistentObject,
                    base.TackerObjectDictCompat):

    # Version 1.0: Initial version
    VERSION = '1.0'

    fields = {
        'id': fields.StringField(nullable=False),
        'operationState': v2fields.LcmOperationStateTypeField(nullable=False),
        'stateEnteredTime': fields.DateTimeField(nullable=False),
        'startTime': fields.DateTimeField(nullable=False),
        'vnfInstanceId': fields.StringField(nullable=False),
        'grantId': fields.StringField(nullable=True),
        'operation': v2fields.LcmOperationTypeField(nullable=False),
        'isAutomaticInvocation': fields.BooleanField(nullable=False),
        'operationParams': OperationParamsField(nullable=True),
        'isCancelPending': fields.BooleanField(nullable=False),
        'cancelMode': v2fields.CancelModeTypeField(nullable=True),
        'error': fields.ObjectField(
            'ProblemDetails', nullable=True),
        'resourceChanges': fields.ObjectField(
            'VnfLcmOpOccV2_ResourceChanges', nullable=True),
        'changedInfo': fields.ObjectField(
            'VnfInfoModificationsV2', nullable=True),
        'changedExtConnectivity': fields.ListOfObjectsField(
            'ExtVirtualLinkInfoV2', nullable=True),
        'modificationsTriggeredByVnfPkgChange': fields.ObjectField(
            'ModificationsTriggeredByVnfPkgChangeV2', nullable=True),
        'vnfSnapshotInfoId': fields.StringField(nullable=True),
        '_links': fields.ObjectField('VnfLcmOpOccV2_Links', nullable=False),
    }


@base.TackerObjectRegistry.register
class VnfLcmOpOccV2_ResourceChanges(base.TackerObject,
                                    base.TackerObjectDictCompat):

    # Version 1.0: Initial version
    VERSION = '1.0'

    fields = {
        'affectedVnfcs': fields.ListOfObjectsField(
            'AffectedVnfcV2', nullable=True),
        'affectedVirtualLinks': fields.ListOfObjectsField(
            'AffectedVirtualLinkV2', nullable=True),
        'affectedExtLinkPorts': fields.ListOfObjectsField(
            'AffectedExtLinkPortV2', nullable=True),
        'affectedVirtualStorages': fields.ListOfObjectsField(
            'AffectedVirtualStorageV2', nullable=True)
    }


@base.TackerObjectRegistry.register
class VnfLcmOpOccV2_Links(base.TackerObject):

    # Version 1.0: Initial version
    VERSION = '1.0'

    fields = {
        'self': fields.ObjectField('Link', nullable=False),
        'vnfInstance': fields.ObjectField('Link', nullable=False),
        'grant': fields.ObjectField('Link', nullable=True),
        'cancel': fields.ObjectField('Link', nullable=True),
        'retry': fields.ObjectField('Link', nullable=True),
        'rollback': fields.ObjectField('Link', nullable=True),
        'fail': fields.ObjectField('Link', nullable=True),
        'vnfSnapshot': fields.ObjectField('Link', nullable=True),
    }



# Copyright 2020 OpenStack Foundation
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

# flake8: noqa: E402

"""change_vnf_filter_column_definition

Revision ID: df26c5871f3c
Revises: 329cd1619d41
Create Date: 2020-11-13 18:32:46.703342

"""

# revision identifiers, used by Alembic.
revision = 'df26c5871f3c'
down_revision = '329cd1619d41'

from alembic import op
import sqlalchemy as sa


from tacker.db import migration


def upgrade(active_plugins=None, options=None):
    # This migration file is to change syntax of "GENERATED ALWAYS AS"
    # from 'filter' to `filter`

    # TODO(esto-aln): (1) Need to fix SQL statement such that "mediumblob"
    # is used instead of "text". Currently, "text" is used as a workaround.
    # (2) Need to fix SQL statement to utilize sqlalchemy. Currently, we
    # use raw SQL with op.exec as a workaround since op.alter_column does
    # not work correctly.
    alter_sql_notification_types = "ALTER TABLE vnf_lcm_filters CHANGE \
        notification_types notification_types text GENERATED \
        ALWAYS AS (json_unquote(json_extract(`filter`,\
        '$.notificationTypes'))) VIRTUAL;"

    alter_sql_notification_types_len = "ALTER TABLE vnf_lcm_filters CHANGE \
        notification_types_len notification_types_len int(11) GENERATED \
        ALWAYS AS (ifnull(json_length(`notification_types`),0)) VIRTUAL;"

    alter_sql_operation_types = "ALTER TABLE vnf_lcm_filters CHANGE \
        operation_types operation_types text GENERATED ALWAYS AS \
        (json_unquote(json_extract(`filter`,'$.operationTypes'))) VIRTUAL;"

    alter_sql_operation_types_len = "ALTER TABLE vnf_lcm_filters CHANGE \
        operation_types_len operation_types_len int(11) GENERATED ALWAYS \
        AS (ifnull(json_length(`operation_types`),0)) VIRTUAL;"

    op.execute(alter_sql_notification_types)
    op.execute(alter_sql_notification_types_len)
    op.execute(alter_sql_operation_types)
    op.execute(alter_sql_operation_types_len)



# Copyright (C) 2020 NTT DATA
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import sys

from oslo_log import log as logging

from glanceclient import exc
from tacker.common import clients
from tacker.extensions import vnflcm


LOG = logging.getLogger(__name__)


class GlanceClient(object):

    def __init__(self, vim_connection_info, version=None):
        super(GlanceClient, self).__init__()
        self.connection = clients.OpenstackSdkConnection(
            vim_connection_info, version).connection

    def create(self, name, **fields):
        try:
            return self.connection.image.create_image(
                name, allow_duplicates=True, **fields)
        except exc.HTTPException:
            type_, value, tb = sys.exc_info()
            raise vnflcm.GlanceClientException(msg=value)

    def delete(self, image_id):
        try:
            self.connection.image.delete_image(image_id)
        except exc.HTTPNotFound:
            LOG.warning("Image %(image)s created not found "
                        "at cleanup", {'image': image_id})

    def import_image(self, image, web_path):
        try:
            self.connection.image.import_image(
                image, method='web-download', uri=web_path)
        except exc.HTTPException:
            type_, value, tb = sys.exc_info()
            raise vnflcm.GlanceClientException(msg=value)

    def get(self, image_id):
        try:
            return self.connection.image.get_image(image_id)
        except exc.HTTPNotFound:
            LOG.warning("Image %(image)s created not found ",
                        {'image': image_id})



# Copyright (C) 2021 Nippon Telegraph and Telephone Corporation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from unittest import mock

from tacker.sol_refactored.api import api_version
from tacker.sol_refactored.common import exceptions as sol_ex
from tacker.tests import base


class TestAPIVersion(base.BaseTestCase):

    def test_init_null(self):
        vers = api_version.APIVersion()
        self.assertTrue(vers.is_null())

    @mock.patch.object(api_version, 'supported_versions',
                  new=["3.1.4159", "2.0.0"])
    def test_init(self):
        for vers, vers_str in [("2.0.0", "2.0.0"),
                               ("3.1.4159", "3.1.4159"),
                               ("2.0.0-impl:foobar", "2.0.0")]:
            v = api_version.APIVersion(vers)
            self.assertEqual(str(v), vers_str)

    def test_init_exceptions(self):
        self.assertRaises(sol_ex.InvalidAPIVersionString,
                          api_version.APIVersion, "0.1.2")

        self.assertRaises(sol_ex.APIVersionNotSupported,
                          api_version.APIVersion, "9.9.9")

    @mock.patch.object(api_version, 'supported_versions',
                  new=["1.3.0", "1.3.1", "2.0.0"])
    def test_compare(self):
        self.assertTrue(api_version.APIVersion("1.3.0") <
                        api_version.APIVersion("1.3.1"))

        self.assertTrue(api_version.APIVersion("2.0.0") >
                        api_version.APIVersion("1.3.1"))

    @mock.patch.object(api_version, 'supported_versions',
                  new=["1.3.0", "1.3.1", "2.0.0"])
    def test_matches(self):
        vers = api_version.APIVersion("2.0.0")
        self.assertTrue(vers.matches(api_version.APIVersion("1.3.0"),
                                     api_version.APIVersion()))

        self.assertFalse(vers.matches(api_version.APIVersion(),
                                      api_version.APIVersion("1.3.1")))



# Copyright 2015 Intel Corporation..
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import abc

from oslo_log import log as logging

from tacker._i18n import _
from tacker.api import extensions
from tacker.api.v1 import attributes as attr
from tacker.api.v1 import base
from tacker.api.v1 import resource_helper
from tacker.common import exceptions
from tacker import manager
from tacker.plugins.common import constants
from tacker.services import service_base


LOG = logging.getLogger(__name__)


class MultipleMGMTDriversSpecified(exceptions.InvalidInput):
    message = _('More than one MGMT Driver per vnfd is not supported')


class ServiceTypesNotSpecified(exceptions.InvalidInput):
    message = _('service types are not specified')


class VNFDInUse(exceptions.InUse):
    message = _('VNFD %(vnfd_id)s is still in use')


class VNFInUse(exceptions.InUse):
    message = _('VNF %(vnf_id)s is still in use')


class InvalidMgmtDriver(exceptions.InvalidInput):
    message = _('Invalid Mgmt driver %(mgmt_driver_name)s.')


class InvalidInfraDriver(exceptions.InvalidInput):
    message = _('VIM type %(vim_name)s is not supported as an infra driver')


class InvalidAPIAttributeType(exceptions.InvalidInput):
    message = _('Expecting dict type for API attribute instead of %(atype)s ')


class VNFCreateFailed(exceptions.TackerException):
    message = _('creating VNF based on %(vnfd_id)s failed')


class VNFUpdateInvalidInput(exceptions.TackerException):
    message = _('VNF Update Invalid Input %(reason)s')


class VNFUpdateWaitFailed(exceptions.TackerException):
    message = _('VNF Update %(reason)s')


class VNFCreateWaitFailed(exceptions.TackerException):
    message = _('VNF Create %(reason)s')


class VNFScaleWaitFailed(exceptions.TackerException):
    message = _('%(reason)s')


class VNFDeleteWaitFailed(exceptions.TackerException):
    message = _('VNF Delete %(reason)s')


class VNFHealWaitFailed(exceptions.TackerException):
    message = _('VNF Heal %(reason)s')


class VNFChangeExtConnWaitFailed(exceptions.TackerException):
    message = _('VNF ChangeExtConn %(reason)s')


class VNFDeleteFailed(exceptions.TackerException):
    message = _('%(reason)s')


class VNFHealFailed(exceptions.TackerException):
    message = _('VNF %(vnf_id)s failed to heal')


class VNFDNotFound(exceptions.NotFound):
    message = _('VNFD %(vnfd_id)s could not be found')


class CnfDefinitionNotFound(exceptions.NotFound):
    message = _(
        "CNF definition file with path %(path)s "
        "is not found in vnf_artifacts.")


class CNFCreateWaitFailed(exceptions.TackerException):
    message = _('CNF Create Failed with reason: %(reason)s')


class CNFScaleFailed(exceptions.TackerException):
    message = _('CNF Scale Failed with reason: %(reason)s')


class CNFScaleWaitFailed(exceptions.TackerException):
    message = _('CNF Scale Wait Failed with reason: %(reason)s')


class CNFHealFailed(exceptions.TackerException):
    message = _('%(reason)s')


class CNFHealWaitFailed(exceptions.TackerException):
    message = _('%(reason)s')


class InvalidVimConnectionInfo(exceptions.TackerException):
    message = _('Invalid vim_connection_info: %(reason)s')


class HelmClientRemoteCommandError(exceptions.TackerException):
    message = _('Failed to execute remote command.')


class HelmClientMissingParamsError(exceptions.TackerException):
    message = _('The specified value %(value)s was not found.')


class HelmClientOtherError(exceptions.TackerException):
    message = _('An error occurred in HelmClient: %(error_message)s.')


class ServiceTypeNotFound(exceptions.NotFound):
    message = _('service type %(service_type_id)s could not be found')


class VNFNotFound(exceptions.NotFound):
    message = _('VNF %(vnf_id)s could not be found')


class LCMUserDataFailed(exceptions.TackerException):
    message = _('LCM user data %(reason)s')


class ParamYAMLNotWellFormed(exceptions.InvalidInput):
    message = _("Parameter YAML not well formed - %(error_msg_details)s")


class ToscaParserFailed(exceptions.InvalidInput):
    message = _("tosca-parser failed: - %(error_msg_details)s")


class HeatTranslatorFailed(exceptions.InvalidInput):
    message = _("heat-translator failed: - %(error_msg_details)s")


class HeatClientException(exceptions.TackerException):
    message = _("%(msg)s")


class IPAddrInvalidInput(exceptions.InvalidInput):
    message = _("IP Address input values should be in a list format")


class HugePageSizeInvalidInput(exceptions.InvalidInput):
    message = _("Value specified for mem_page_size is invalid: "
                "%(error_msg_details)s. The valid values are 'small', 'large',"
                " 'any' or an integer value in MB")


class CpuAllocationInvalidKeys(exceptions.InvalidInput):
    message = _("Invalid keys specified in VNFD - %(error_msg_details)s."
                "Supported keys are: %(valid_keys)s")


class CpuAllocationInvalidValues(exceptions.InvalidInput):
    message = _("Invalid values specified in VNFD - %(error_msg_details)s."
                "Supported Values are: %(valid_values)s")


class NumaNodesInvalidKeys(exceptions.InvalidInput):
    message = _("Invalid keys specified in VNFD - %(error_msg_details)s."
                "Supported keys are: %(valid_keys)s")


class FilePathMissing(exceptions.InvalidInput):
    message = _("'file' attribute is missing for "
                "tosca.artifacts.Deployment.Image.VM artifact type")


class InfraDriverUnreachable(exceptions.ServiceUnavailable):
    message = _("Could not retrieve VNF resource IDs and"
                " types. Please check %(service)s status.")


class VNFInactive(exceptions.InvalidInput):
    message = _("VNF %(vnf_id)s is not in Active state: %(message)s")


class MetadataNotMatched(exceptions.InvalidInput):
    message = _("Metadata for alarm policy is not matched")


class InvalidResourceType(exceptions.InvalidInput):
    message = _("Resource type %(resource_type)s for alarm policy "
                "is not supported")


class InvalidSubstitutionMapping(exceptions.InvalidInput):
    message = _("Input for substitution mapping requirements are not"
                " valid for %(requirement)s. They must be in the form"
                " of list with two entries")


class SMRequirementMissing(exceptions.InvalidInput):
    message = _("All the requirements for substitution_mappings are not"
                " provided. Missing requirement for %(requirement)s")


class InvalidParamsForSM(exceptions.InvalidInput):
    message = _("Please provide parameters for substitution mappings")


class InvalidKubernetesScalingPolicyNumber(exceptions.InvalidInput):
    message = _("Please provide only one Scaling policy")


class InvalidKubernetesNetworkNumber(exceptions.InvalidInput):
    message = _("Please provide one network for all vdus")


class InvalidKubernetesInputParameter(exceptions.InvalidInput):
    message = _("Found unsupported keys for %(found_keys)s ")


class InvalidInstReqInfoForScaling(exceptions.InvalidInput):
    message = _("Scaling resource cannot be set to "
                "fixed ip_address or mac_address.")


class InvalidMaintenanceParameter(exceptions.InvalidInput):
    message = _("Could not find the required params for maintenance")


def _validate_service_type_list(data, valid_values=None):
    if not isinstance(data, list):
        msg = _("Invalid data format for service list: '%s'") % data
        LOG.debug(msg)
        return msg
    if not data:
        msg = _("Empty list is not allowed for service list. '%s'") % data
        LOG.debug(msg)
        return msg
    key_specs = {
        'service_type': {
            'type:string': None,
        }
    }
    for service in data:
        msg = attr._validate_dict(service, key_specs)
        if msg:
            LOG.debug(msg)
            return msg


attr.validators['type:service_type_list'] = _validate_service_type_list

NAME_MAX_LEN = 255

RESOURCE_ATTRIBUTE_MAP = {

    'vnfds': {
        'id': {
            'allow_post': False,
            'allow_put': False,
            'validate': {'type:uuid': None},
            'is_visible': True,
            'primary_key': True,
        },
        'tenant_id': {
            'allow_post': True,
            'allow_put': False,
            'validate': {'type:string': None},
            'required_by_policy': True,
            'is_visible': True,
        },
        'name': {
            'allow_post': True,
            'allow_put': True,
            'validate': {'type:string': NAME_MAX_LEN},
            'is_visible': True,
        },
        'description': {
            'allow_post': True,
            'allow_put': True,
            'validate': {'type:string': None},
            'is_visible': True,
            'default': '',
        },
        'service_types': {
            'allow_post': True,
            'allow_put': False,
            'convert_to': attr.convert_to_list,
            'validate': {'type:service_type_list': None},
            'is_visible': True,
            'default': attr.ATTR_NOT_SPECIFIED,
        },
        'attributes': {
            'allow_post': True,
            'allow_put': False,
            'convert_to': attr.convert_none_to_empty_dict,
            'validate': {'type:dict_or_nodata': None},
            'is_visible': True,
            'default': None,
        },
        'created_at': {
            'allow_post': False,
            'allow_put': False,
            'is_visible': True,
        },
        'updated_at': {
            'allow_post': False,
            'allow_put': False,
            'is_visible': True,
        },
        'template_source': {
            'allow_post': False,
            'allow_put': False,
            'is_visible': True,
            'default': 'onboarded'
        },
    },

    'vnfs': {
        'id': {
            'allow_post': False,
            'allow_put': False,
            'validate': {'type:uuid': None},
            'is_visible': True,
            'primary_key': True
        },
        'tenant_id': {
            'allow_post': True,
            'allow_put': False,
            'validate': {'type:string': None},
            'required_by_policy': True,
            'is_visible': True
        },
        'vnfd_id': {
            'allow_post': True,
            'allow_put': False,
            'validate': {'type:uuid': None},
            'is_visible': True,
            'default': None
        },
        'vim_id': {
            'allow_post': True,
            'allow_put': False,
            'validate': {'type:string': None},
            'is_visible': True,
            'default': '',
        },
        'name': {
            'allow_post': True,
            'allow_put': True,
            'validate': {'type:string': NAME_MAX_LEN},
            'is_visible': True,
        },
        'description': {
            'allow_post': True,
            'allow_put': True,
            'validate': {'type:string': None},
            'is_visible': True,
            'default': '',
        },
        'instance_id': {
            'allow_post': False,
            'allow_put': False,
            'validate': {'type:string': None},
            'is_visible': True,
        },
        'mgmt_ip_address': {
            'allow_post': False,
            'allow_put': False,
            'validate': {'type:string': None},
            'is_visible': True,
        },
        'attributes': {
            'allow_post': True,
            'allow_put': True,
            'validate': {'type:dict_or_none': None},
            'is_visible': True,
            'default': {},
        },
        'placement_attr': {
            'allow_post': True,
            'allow_put': False,
            'validate': {'type:dict_or_none': None},
            'is_visible': True,
            'default': {},
        },
        'status': {
            'allow_post': False,
            'allow_put': False,
            'is_visible': True,
        },
        'error_reason': {
            'allow_post': False,
            'allow_put': False,
            'is_visible': True,
        },
        'created_at': {
            'allow_post': False,
            'allow_put': False,
            'is_visible': True,
        },
        'updated_at': {
            'allow_post': False,
            'allow_put': False,
            'is_visible': True,
        },
        'vnfd_template': {
            'allow_post': True,
            'allow_put': False,
            'validate': {'type:dict_or_none': None},
            'is_visible': True,
            'default': None,
        },
    },
}


SUB_RESOURCE_ATTRIBUTE_MAP = {
    'actions': {
        'parent': {
            'collection_name': 'vnfs',
            'member_name': 'vnf'
        },
        'members': {
            'scale': {
                'parameters': {
                    'policy': {
                        'allow_post': True,
                        'allow_put': False,
                        'is_visible': True,
                        'validate': {'type:string': None}
                    },
                    'type': {
                        'allow_post': True,
                        'allow_put': False,
                        'is_visible': True,
                        'validate': {'type:string': None}
                    },
                    'tenant_id': {
                        'allow_post': True,
                        'allow_put': False,
                        'validate': {'type:string': None},
                        'required_by_policy': False,
                        'is_visible': False
                    },
                }
            },
        }
    },
    'triggers': {
        'parent': {
            'collection_name': 'vnfs',
            'member_name': 'vnf'
        },
        'members': {
            'trigger': {
                'parameters': {
                    'policy_name': {
                        'allow_post': True,
                        'allow_put': False,
                        'is_visible': True,
                        'validate': {'type:string': None}
                    },
                    'action_name': {
                        'allow_post': True,
                        'allow_put': False,
                        'is_visible': True,
                        'validate': {'type:string': None}
                    },
                    'params': {
                        'allow_post': True,
                        'allow_put': False,
                        'is_visible': True,
                        'validate': {'type:dict_or_none': None}
                    },
                    'tenant_id': {
                        'allow_post': True,
                        'allow_put': False,
                        'validate': {'type:string': None},
                        'required_by_policy': False,
                        'is_visible': False
                    }
                }
            },
        }
    },
    'resources': {
        'parent': {
            'collection_name': 'vnfs',
            'member_name': 'vnf'
        },
        'members': {
            'resource': {
                'parameters': {
                    'name': {
                        'allow_post': False,
                        'allow_put': False,
                        'is_visible': True,
                    },
                    'type': {
                        'allow_post': False,
                        'allow_put': False,
                        'is_visible': True,
                    },
                    'id': {
                        'allow_post': False,
                        'allow_put': False,
                        'is_visible': True,
                    },
                }
            }
        }
    },
    'maintenances': {
        'parent': {
            'collection_name': 'vnfs',
            'member_name': 'vnf'
        },
        'members': {
            'maintenance': {
                'parameters': {
                    'params': {
                        'allow_post': True,
                        'allow_put': False,
                        'is_visible': True,
                        'validate': {'type:dict_or_none': None}
                    },
                    'tenant_id': {
                        'allow_post': True,
                        'allow_put': False,
                        'validate': {'type:string': None},
                        'required_by_policy': False,
                        'is_visible': False
                    },
                    'response': {
                        'allow_post': False,
                        'allow_put': False,
                        'validate': {'type:dict_or_none': None},
                        'is_visible': True
                    }
                }
            }
        }
    }
}


class Vnfm(extensions.ExtensionDescriptor):
    @classmethod
    def get_name(cls):
        return 'VNF Manager'

    @classmethod
    def get_alias(cls):
        return 'VNFM'

    @classmethod
    def get_description(cls):
        return "Extension for VNF Manager"

    @classmethod
    def get_namespace(cls):
        return 'https://wiki.openstack.org/Tacker'

    @classmethod
    def get_updated(cls):
        return "2013-11-19T10:00:00-00:00"

    @classmethod
    def get_resources(cls):
        special_mappings = {}
        plural_mappings = resource_helper.build_plural_mappings(
            special_mappings, RESOURCE_ATTRIBUTE_MAP)
        plural_mappings['service_types'] = 'service_type'
        attr.PLURALS.update(plural_mappings)
        resources = resource_helper.build_resource_info(
            plural_mappings, RESOURCE_ATTRIBUTE_MAP, constants.VNFM,
            translate_name=True)
        plugin = manager.TackerManager.get_service_plugins()[
            constants.VNFM]
        for collection_name in SUB_RESOURCE_ATTRIBUTE_MAP:
            parent = SUB_RESOURCE_ATTRIBUTE_MAP[collection_name]['parent']

            for resource_name in SUB_RESOURCE_ATTRIBUTE_MAP[
                    collection_name]['members']:
                params = SUB_RESOURCE_ATTRIBUTE_MAP[
                    collection_name]['members'][resource_name]['parameters']

                controller = base.create_resource(collection_name,
                                                  resource_name,
                                                  plugin, params,
                                                  allow_bulk=True,
                                                  parent=parent)

                resource = extensions.ResourceExtension(
                    collection_name,
                    controller, parent,
                    attr_map=params)
                resources.append(resource)
        return resources

    @classmethod
    def get_plugin_interface(cls):
        return VNFMPluginBase

    def update_attributes_map(self, attributes):
        super(Vnfm, self).update_attributes_map(
            attributes, extension_attrs_map=RESOURCE_ATTRIBUTE_MAP)

    def get_extended_resources(self, version):
        version_map = {'1.0': RESOURCE_ATTRIBUTE_MAP}
        return version_map.get(version, {})


class VNFMPluginBase(service_base.NFVPluginBase, metaclass=abc.ABCMeta):
    def get_plugin_name(self):
        return constants.VNFM

    def get_plugin_type(self):
        return constants.VNFM

    def get_plugin_description(self):
        return 'Tacker VNF Manager plugin'

    @abc.abstractmethod
    def create_vnfd(self, context, vnfd):
        pass

    @abc.abstractmethod
    def delete_vnfd(self, context, vnfd_id):
        pass

    @abc.abstractmethod
    def get_vnfd(self, context, vnfd_id, fields=None):
        pass

    @abc.abstractmethod
    def get_vnfds(self, context, filters=None, fields=None):
        pass

    @abc.abstractmethod
    def get_vnfs(self, context, filters=None, fields=None):
        pass

    @abc.abstractmethod
    def get_vnf(self, context, vnf_id, fields=None):
        pass

    @abc.abstractmethod
    def get_vnf_resources(self, context, vnf_id, fields=None, filters=None):
        pass

    @abc.abstractmethod
    def create_vnf(self, context, vnf):
        pass

    @abc.abstractmethod
    def update_vnf(
            self, context, vnf_id, vnf):
        pass

    @abc.abstractmethod
    def delete_vnf(self, context, vnf_id):
        pass

    @abc.abstractmethod
    def create_vnf_scale(
            self, context, vnf_id, scale):
        pass

    @abc.abstractmethod
    def create_vnf_trigger(
            self, context, vnf_id, trigger):
        pass

    @abc.abstractmethod
    def create_vnf_maintenance(self, context, vnf_id, maintenance):
        pass
